from unittest import TestCase, main, mock
from schema import SchemaError
import copy
import argparse
import skelebot as sb

class TestRepository(TestCase):

    s3Repo = None
    artifcatory = None
    s3 = None
    s3_subfolder = None

    artifactoryDict = {
        "url": "test",
        "repo": "test",
        "path": "path"
    }

    s3Dict = {
        "bucket": "my-bucket",
        "region": "us-east-1",
        "profile": "test"
    }

    artifactDict = {
        "name": "test",
        "file": "test"
    }

    def setUp(self):
        artifact = sb.components.repository.repository.Artifact("test", "test.pkl")
        artifact2 = sb.components.repository.repository.Artifact("test2", "test2.pkl")
        artifact3 = sb.components.repository.repository.Artifact("test3", "test3.pkl", singular=True)
        artifactoryRepo = sb.components.repository.artifactoryRepo.ArtifactoryRepo("artifactory.test.com", "ml", "test")
        s3Repo = sb.components.repository.s3Repo.S3Repo("my-bucket", "us-east-1", "test")
        s3Repo_path = sb.components.repository.s3Repo.S3Repo("my-bucket/sub/folder", "us-east-1")

        self.s3Repo = s3Repo
        self.artifactory = sb.components.repository.repository.Repository([artifact, artifact2, artifact3], s3=None, artifactory=artifactoryRepo)
        self.s3 = sb.components.repository.repository.Repository([artifact, artifact2, artifact3], s3=s3Repo, artifactory=None)
        self.s3_subfolder = sb.components.repository.repository.Repository([artifact, artifact3], s3=s3Repo_path, artifactory=None)

    def test_repository_load(self):
        artifact = sb.components.repository.repository.Artifact("test", "test.pkl")

        sb.components.repository.repository.Repository.load({"artifacts": [], "s3": self.s3Dict})
        sb.components.repository.repository.Repository.load({"artifacts": [], "artifactory": self.artifactoryDict})

        try:
            sb.components.repository.repository.Repository.load({"artifacts": []})
            self.fail("Exception Not Thrown")
        except SchemaError as exc:
            self.assertEqual(str(exc), "Repository must contain 's3' or 'artifactory' config")

    def test_addParsers_artifactory(self):
        parser = argparse.ArgumentParser(formatter_class=argparse.RawTextHelpFormatter)
        subparsers = parser.add_subparsers(dest="job")
        subparsers = self.artifactory.addParsers(subparsers)

        self.assertNotEqual(subparsers.choices["push"], None)
        self.assertNotEqual(subparsers.choices["pull"], None)

    def test_addParsers_s3(self):
        parser = argparse.ArgumentParser(formatter_class=argparse.RawTextHelpFormatter)
        subparsers = parser.add_subparsers(dest="job")
        subparsers = self.s3.addParsers(subparsers)

        self.assertNotEqual(subparsers.choices["push"], None)
        self.assertNotEqual(subparsers.choices["pull"], None)

    @mock.patch('skelebot.components.repository.artifactoryRepo.getpass')
    @mock.patch('skelebot.components.repository.artifactoryRepo.input')
    @mock.patch('os.rename')
    @mock.patch('artifactory.ArtifactoryPath')
    def test_execute_push_conflict_artifactory(self, mock_artifactory, mock_rename, mock_input, mock_pass):
        mock_input.return_value = "abc"
        mock_pass.return_value = "abc"
        config = sb.objects.config.Config(version="1.0.0")
        args = argparse.Namespace(job="push", force=False, artifact='test', user=None, token=None, prefix=None)
        expectedException = "This artifact version already exists. Please bump the version or use the force parameter (-f) to overwrite the artifact."

        try:
            self.artifactory.execute(config, args)
            self.fail("Exception Not Thrown")
        except Exception as exc:
            self.assertEqual(str(exc), expectedException)
            mock_artifactory.assert_called_with("artifactory.test.com/ml/test/test_v1.0.0.pkl", auth=('abc', 'abc'))

    @mock.patch('boto3.Session')
    def test_execute_push_conflict_s3(self, mock_boto3_session):
        mock_client = mock.Mock()
        mock_session = mock.Mock()
        mock_client.list_objects_v2.return_value = {"Contents": [{"Key": "test_v1.0.0.pkl"}]}
        mock_session.client.return_value = mock_client
        mock_boto3_session.return_value = mock_session

        config = sb.objects.config.Config(version="1.0.0")
        args = argparse.Namespace(job="push", force=False, artifact='test', user='sean', token='abc123', prefix=None)
        expectedException = "This artifact version already exists. Please bump the version or use the force parameter (-f) to overwrite the artifact."

        try:
            self.s3.execute(config, args)
            self.fail("Exception Not Thrown")
        except Exception as exc:
            self.assertEqual(str(exc), expectedException)
            mock_client.list_objects_v2.assert_called_with(Bucket="my-bucket", Prefix="test_v1.0.0.pkl")

    @mock.patch('shutil.copyfile')
    @mock.patch('os.remove')
    @mock.patch('artifactory.ArtifactoryPath')
    def test_execute_push_error_artifactory(self, mock_artifactory, mock_remove, mock_copy):
        mock_path = mock.MagicMock()
        mock_path.deploy_file = mock.MagicMock(side_effect=KeyError('foo'))
        mock_artifactory.return_value = mock_path

        config = sb.objects.config.Config(version="1.0.0")
        args = argparse.Namespace(job="push", force=True, artifact='test', user='sean', token='abc123', prefix=None)

        with self.assertRaises(KeyError):
            self.artifactory.execute(config, args)

        mock_artifactory.assert_called_with("artifactory.test.com/ml/test/test_v1.0.0.pkl", auth=('sean', 'abc123'))
        mock_copy.assert_called_with("test.pkl", "test_v1.0.0.pkl")
        mock_remove.assert_called_with("test_v1.0.0.pkl")

    @mock.patch('shutil.copyfile')
    @mock.patch('os.remove')
    @mock.patch('artifactory.ArtifactoryPath')
    def test_execute_push_artifactory(self, mock_artifactory, mock_remove, mock_copy):
        config = sb.objects.config.Config(version="1.0.0")
        args = argparse.Namespace(job="push", force=True, artifact='test', user='sean', token='abc123', prefix=None)

        self.artifactory.execute(config, args)

        mock_artifactory.assert_called_with("artifactory.test.com/ml/test/test_v1.0.0.pkl", auth=('sean', 'abc123'))
        mock_copy.assert_called_with("test.pkl", "test_v1.0.0.pkl")
        mock_remove.assert_called_with("test_v1.0.0.pkl")

    @mock.patch('shutil.copyfile')
    @mock.patch('os.remove')
    @mock.patch('artifactory.ArtifactoryPath')
    def test_execute_push_artifactory_prefix(self, mock_artifactory, mock_remove, mock_copy):
        config = sb.objects.config.Config(version="1.0.0")
        args = argparse.Namespace(job="push", force=True, artifact='test', user='sean', token='abc123', prefix="prefix_")

        self.artifactory.execute(config, args)

        mock_artifactory.assert_called_with("artifactory.test.com/ml/test/prefix_test_v1.0.0.pkl", auth=('sean', 'abc123'))
        mock_copy.assert_called_with("test.pkl", "prefix_test_v1.0.0.pkl")
        mock_remove.assert_called_with("prefix_test_v1.0.0.pkl")

    @mock.patch('shutil.copyfile')
    @mock.patch('os.remove')
    @mock.patch('artifactory.ArtifactoryPath')
    def test_execute_push_artifactory_singular(self, mock_artifactory, mock_remove, mock_copy):
        config = sb.objects.config.Config(version="1.0.0")
        args = argparse.Namespace(job="push", force=True, artifact='test3', user='sean', token='abc123', prefix=None)

        self.artifactory.execute(config, args)

        mock_artifactory.assert_called_with("artifactory.test.com/ml/test/test3.pkl", auth=('sean', 'abc123'))
        mock_copy.assert_called_with("test3.pkl", "test3.pkl")
        mock_remove.assert_called_with("test3.pkl")

    @mock.patch('boto3.Session')
    def test_execute_push_s3(self, mock_boto3_session):
        mock_client = mock.Mock()
        mock_session = mock.Mock()
        mock_session.client.return_value = mock_client
        mock_boto3_session.return_value = mock_session

        config = sb.objects.config.Config(version="1.0.0")
        args = argparse.Namespace(job="push", force=True, artifact='test', user='sean', token='abc123', prefix=None)

        self.s3.execute(config, args)
        mock_client.upload_file.assert_called_with("test.pkl", "my-bucket", "test_v1.0.0.pkl")

    @mock.patch('boto3.Session')
    def test_execute_push_s3_prefix(self, mock_boto3_session):
        mock_client = mock.Mock()
        mock_session = mock.Mock()
        mock_session.client.return_value = mock_client
        mock_boto3_session.return_value = mock_session

        config = sb.objects.config.Config(version="1.0.0")
        args = argparse.Namespace(job="push", force=True, artifact='test', user='sean', token='abc123', prefix="prefix_")

        self.s3.execute(config, args)
        mock_client.upload_file.assert_called_with("test.pkl", "my-bucket", "prefix_test_v1.0.0.pkl")

    @mock.patch('boto3.Session')
    def test_execute_push_s3_singular(self, mock_boto3_session):
        mock_client = mock.Mock()
        mock_session = mock.Mock()
        mock_session.client.return_value = mock_client
        mock_boto3_session.return_value = mock_session

        config = sb.objects.config.Config(version="1.0.0")
        args = argparse.Namespace(job="push", force=True, artifact='test3', user='sean', token='abc123', prefix=None)

        self.s3.execute(config, args)
        mock_client.upload_file.assert_called_with("test3.pkl", "my-bucket", "test3.pkl")

    @mock.patch('shutil.copyfile')
    @mock.patch('os.remove')
    @mock.patch('artifactory.ArtifactoryPath')
    def test_execute_push_artifactory_all(self, mock_artifactory, mock_remove, mock_copy):
        config = sb.objects.config.Config(version="1.0.0")
        args = argparse.Namespace(job="push", force=True, artifact='ALL', user='sean', token='abc123', prefix=None)

        self.artifactory.execute(config, args)

        mock_artifactory.assert_has_calls([
            mock.call("artifactory.test.com/ml/test/test_v1.0.0.pkl", auth=('sean', 'abc123')),
            mock.call("artifactory.test.com/ml/test/test2_v1.0.0.pkl", auth=('sean', 'abc123'))
        ], any_order=True)
        mock_copy.assert_has_calls([
            mock.call("test.pkl", "test_v1.0.0.pkl"),
            mock.call("test2.pkl", "test2_v1.0.0.pkl")
        ], any_order=True)
        mock_remove.assert_has_calls([
            mock.call("test_v1.0.0.pkl"),
            mock.call("test2_v1.0.0.pkl")
        ], any_order=True)

    @mock.patch('boto3.Session')
    def test_execute_push_s3_all(self, mock_boto3_session):
        mock_client = mock.Mock()
        mock_session = mock.Mock()
        mock_session.client.return_value = mock_client
        mock_boto3_session.return_value = mock_session

        config = sb.objects.config.Config(version="1.0.0")
        args = argparse.Namespace(job="push", force=True, artifact='ALL', user='sean', token='abc124', prefix=None)

        self.s3.execute(config, args)
        mock_client.upload_file.assert_has_calls([
            mock.call("test.pkl", "my-bucket", "test_v1.0.0.pkl"),
            mock.call("test2.pkl", "my-bucket", "test2_v1.0.0.pkl")
        ], any_order=True)

    @mock.patch('boto3.Session')
    def test_execute_push_s3_subfolder(self, mock_boto3_session):
        mock_client = mock.Mock()
        mock_session = mock.Mock()
        mock_session.client.return_value = mock_client
        mock_boto3_session.return_value = mock_session

        config = sb.objects.config.Config(version="1.0.0")
        args = argparse.Namespace(job="push", force=True, artifact='test', user='sean', token='abc123', prefix=None)

        self.s3_subfolder.execute(config, args)
        mock_client.upload_file.assert_called_with("test.pkl", "my-bucket", "sub/folder/test_v1.0.0.pkl")

    @mock.patch('skelebot.components.repository.artifactoryRepo.getpass')
    @mock.patch('skelebot.components.repository.artifactoryRepo.input')
    @mock.patch('builtins.open')
    @mock.patch('artifactory.ArtifactoryPath')
    def test_execute_pull_artifactory(self, mock_artifactory, mock_open, mock_input, mock_pass):
        mock_input.return_value = "abc"
        mock_pass.return_value = "abc"

        config = sb.objects.config.Config(version="1.0.0")
        args = argparse.Namespace(job="pull", version='0.1.0', artifact='test', user=None, token=None, override=False)

        self.artifactory.execute(config, args)

        mock_artifactory.assert_called_with("artifactory.test.com/ml/test/test_v0.1.0.pkl", auth=("abc", "abc"))
        mock_open.assert_called_with("test_v0.1.0.pkl", "wb")

    @mock.patch('skelebot.components.repository.artifactoryRepo.getpass')
    @mock.patch('skelebot.components.repository.artifactoryRepo.input')
    @mock.patch('builtins.open')
    @mock.patch('artifactory.ArtifactoryPath')
    def test_execute_pull_artifactory_singular(self, mock_artifactory, mock_open, mock_input, mock_pass):
        mock_input.return_value = "abc"
        mock_pass.return_value = "abc"

        config = sb.objects.config.Config(version="1.0.0")
        args = argparse.Namespace(job="pull", version='0.1.0', artifact='test3', user=None, token=None, override=False)

        self.artifactory.execute(config, args)

        mock_artifactory.assert_called_with("artifactory.test.com/ml/test/test3.pkl", auth=("abc", "abc"))
        mock_open.assert_called_with("test3.pkl", "wb")

    @mock.patch('boto3.Session')
    def test_execute_pull_s3(self, mock_boto3_session):
        mock_client = mock.Mock()
        mock_session = mock.Mock()
        mock_session.client.return_value = mock_client
        mock_boto3_session.return_value = mock_session

        config = sb.objects.config.Config(version="1.0.0")
        args = argparse.Namespace(job="pull", version='0.1.0', artifact='test', user=None, token=None, override=False)

        self.s3.execute(config, args)
        mock_client.download_file.assert_called_with("my-bucket", "test_v0.1.0.pkl", "test_v0.1.0.pkl")

    @mock.patch('boto3.Session')
    def test_execute_pull_s3_subfolder(self, mock_boto3_session):
        mock_client = mock.Mock()
        mock_session = mock.Mock()
        mock_session.client.return_value = mock_client
        mock_boto3_session.return_value = mock_session

        config = sb.objects.config.Config(version="1.0.0")
        args = argparse.Namespace(job="pull", version='0.1.0', artifact='test', user=None, token=None, override=False)

        self.s3_subfolder.execute(config, args)
        mock_client.download_file.assert_called_with("my-bucket", "sub/folder/test_v0.1.0.pkl", "test_v0.1.0.pkl")

    @mock.patch('boto3.Session')
    def test_execute_pull_s3_subfolder_singular(self, mock_boto3_session):
        mock_client = mock.Mock()
        mock_session = mock.Mock()
        mock_session.client.return_value = mock_client
        mock_boto3_session.return_value = mock_session

        config = sb.objects.config.Config(version="1.0.0")
        args = argparse.Namespace(job="pull", version='0.1.0', artifact='test3', user=None, token=None, override=False)

        self.s3_subfolder.execute(config, args)
        mock_client.download_file.assert_called_with("my-bucket", "sub/folder/test3.pkl", "test3.pkl")

    @mock.patch('skelebot.components.repository.artifactoryRepo.getpass')
    @mock.patch('skelebot.components.repository.artifactoryRepo.input')
    @mock.patch('builtins.open')
    @mock.patch('artifactory.ArtifactoryPath')
    def test_execute_pull_lcv_artifactory(self, mock_artifactory, mock_open, mock_input, mock_pass):
        mock_apath = mock_artifactory.return_value
        mock_input.return_value = "abc"
        mock_pass.return_value = "abc"
        mock_apath.__iter__.return_value = ["test_v1.1.0", "test_v0.2.4", "test_v1.0.0", "test_v2.0.1"]

        config = sb.objects.config.Config(version="1.0.9")
        args = argparse.Namespace(job="pull", version='LATEST', artifact='test', user=None, token=None, override=False)

        self.artifactory.execute(config, args)

        mock_artifactory.assert_called_with("artifactory.test.com/ml/test/test_v1.0.0.pkl", auth=("abc", "abc"))
        mock_open.assert_called_with("test_v1.0.0.pkl", "wb")

    @mock.patch('boto3.Session')
    def test_execute_pull_lcv_s3(self, mock_boto3_session):
        mock_client = mock.Mock()
        mock_session = mock.Mock()
        mock_client.list_objects_v2.return_value = {"Contents": [{"Key": "test_v1.1.0.pkl"},{"Key": "test_v1.0.5.pkl"},{"Key": "test_v1.0.0.pkl"}]}
        mock_session.client.return_value = mock_client
        mock_boto3_session.return_value = mock_session

        config = sb.objects.config.Config(version="1.0.9")
        args = argparse.Namespace(job="pull", version='LATEST', artifact='test', user=None, token=None, override=False)

        self.s3.execute(config, args)

        mock_client.list_objects_v2.assert_called_with(Bucket="my-bucket", Prefix="test_v1")
        mock_client.download_file.assert_called_with("my-bucket", "test_v1.0.5.pkl", "test_v1.0.5.pkl")

    @mock.patch('boto3.Session')
    def test_execute_pull_s3_in_mem(self, mock_boto3_session):
        mock_client = mock.Mock()
        mock_session = mock.Mock()
        mock_streambody = mock.Mock()
        mock_client.list_objects_v2.return_value = {"Contents": [{"Key": "test_v1.1.0.pkl"},{"Key": "test_v1.0.5.pkl"},{"Key": "test_v1.0.0.pkl"}]}
        mock_session.client.return_value = mock_client
        mock_client.get_object.return_value = {"Body": mock_streambody}
        mock_boto3_session.return_value = mock_session

        self.s3Repo.pull(sb.components.repository.repository.Artifact("test", "test.pkl"), "LATEST", currentVersion="1.0.5", in_memory=True)

        mock_client.list_objects_v2.assert_called_with(Bucket="my-bucket", Prefix="test_v1")
        mock_client.get_object.assert_called_with(Bucket="my-bucket", Key="test_v1.0.5.pkl")
        mock_streambody.read.assert_called_once()

    @mock.patch('skelebot.components.repository.artifactoryRepo.getpass')
    @mock.patch('skelebot.components.repository.artifactoryRepo.input')
    @mock.patch('builtins.open')
    @mock.patch('artifactory.ArtifactoryPath')
    def test_execute_pull_lcv_not_found_artifactory(self, mock_artifactory, mock_open, mock_input, mock_pass):
        mock_apath = mock_artifactory.return_value
        mock_input.return_value = "abc"
        mock_pass.return_value = "abc"
        mock_apath.__iter__.return_value = ["test_v1.1.0", "test_v0.2.4", "test_v1.0.0", "test_v2.0.1"]

        config = sb.objects.config.Config(version="3.0.9")
        args = argparse.Namespace(job="pull", version='LATEST', artifact='test', user=None, token=None, override=False)

        try:
            self.artifactory.execute(config, args)
            self.fail("Exception Not Thrown")
        except RuntimeError as err:
            self.assertEqual(str(err), "No Compatible Version Found")

    @mock.patch('boto3.Session')
    def test_execute_pull_lcv_not_found_s3(self, mock_boto3_session):
        mock_client = mock.Mock()
        mock_session = mock.Mock()
        mock_client.list_objects_v2.return_value = {"Contents": [{"Key": "test_v1.1.0.pkl"},{"Key": "test_v1.0.5.pkl"},{"Key": "test_v1.0.0.pkl"}]}
        mock_session.client.return_value = mock_client
        mock_boto3_session.return_value = mock_session

        config = sb.objects.config.Config(version="2.0.9")
        args = argparse.Namespace(job="pull", version='LATEST', artifact='test', user=None, token=None, override=False)

        try:
            self.s3.execute(config, args)
            self.fail("Exception Not Thrown")
        except RuntimeError as err:
            self.assertEqual(str(err), "No Compatible Version Found")

    @mock.patch('skelebot.components.repository.artifactoryRepo.getpass')
    @mock.patch('skelebot.components.repository.artifactoryRepo.input')
    @mock.patch('builtins.open')
    @mock.patch('artifactory.ArtifactoryPath')
    def test_execute_pull_override_and_lcv_artifactory(self, mock_artifactory, mock_open, mock_input, mock_pass):
        mock_apath = mock_artifactory.return_value
        mock_input.return_value = "abc"
        mock_pass.return_value = "abc"
        mock_apath.__iter__.return_value = ["test_v1.1.0", "test_v0.2.4", "test_v1.0.0", "test_v2.0.1"]

        config = sb.objects.config.Config(version="0.6.9")
        args = argparse.Namespace(job="pull", version='LATEST', artifact='test', user=None, token=None, override=True)

        self.artifactory.execute(config, args)

        mock_artifactory.assert_called_with("artifactory.test.com/ml/test/test_v0.2.4.pkl", auth=("abc", "abc"))
        mock_open.assert_called_with("test.pkl", "wb")

    @mock.patch('boto3.Session')
    def test_execute_pull_override_and_lcv_s3(self, mock_boto3_session):
        mock_client = mock.Mock()
        mock_session = mock.Mock()
        mock_client.list_objects_v2.return_value = {"Contents": [{"Key": "test_v1.1.0.pkl"},{"Key": "test_v1.0.5.pkl"},{"Key": "test_v1.0.0.pkl"}]}
        mock_session.client.return_value = mock_client
        mock_boto3_session.return_value = mock_session

        config = sb.objects.config.Config(version="1.0.3")
        args = argparse.Namespace(job="pull", version='LATEST', artifact='test', user=None, token=None, override=True)

        self.s3.execute(config, args)

        mock_client.download_file.assert_called_with("my-bucket", "test_v1.0.0.pkl", "test.pkl")

    @mock.patch('skelebot.components.repository.artifactoryRepo.getpass')
    @mock.patch('skelebot.components.repository.artifactoryRepo.input')
    @mock.patch('artifactory.ArtifactoryPath')
    def test_execute_pull_not_found(self, mock_artifactory, mock_input, mock_pass):
        mock_input.return_value = "abc"
        mock_pass.return_value = "abc"
        path = mock_artifactory.return_value
        path.exists.return_value = False

        config = sb.objects.config.Config(version="1.0.0")
        args = argparse.Namespace(job="pull", version='0.1.0', artifact='test', user=None, token=None, override=False)

        self.artifactory.execute(config, args)

        mock_artifactory.assert_called_with("artifactory.test.com/ml/test/test_v0.1.0.pkl", auth=("abc", "abc"))

    def test_validate_valid(self):
        try:
            sb.components.repository.artifactoryRepo.ArtifactoryRepo.validate(self.artifactoryDict)
        except:
            self.fail("Validation Raised Exception Unexpectedly")

        try:
            sb.components.repository.s3Repo.S3Repo.validate(self.s3Dict)
        except:
            self.fail("Validation Raised Exception Unexpectedly")

        try:
            sb.components.repository.repository.Artifact.validate(self.artifactDict)
        except:
            self.fail("Validation Raised Exception Unexpectedly")

    def test_validate_missing(self):
        s3Dict = copy.deepcopy(self.s3Dict)
        del s3Dict['bucket']
        del s3Dict['region']
        del s3Dict['profile']

        try:
            sb.components.repository.s3Repo.S3Repo.validate(s3Dict)
        except SchemaError as error:
            self.assertEqual(error.code, "Missing keys: 'bucket', 'region'")

        artifactoryDict = copy.deepcopy(self.artifactoryDict)
        del artifactoryDict['url']
        del artifactoryDict['repo']
        del artifactoryDict['path']

        try:
            sb.components.repository.artifactoryRepo.ArtifactoryRepo.validate(artifactoryDict)
        except SchemaError as error:
            self.assertEqual(error.code, "Missing keys: 'path', 'repo', 'url'")

        artifactDict = copy.deepcopy(self.artifactDict)
        del artifactDict['name']
        del artifactDict['file']

        try:
            sb.components.repository.repository.Artifact.validate(artifactDict)
        except SchemaError as error:
            self.assertEqual(error.code, "Missing keys: 'file', 'name'")

    def validate_error_s3(self, attr, reset, expected):
        s3Dict = copy.deepcopy(self.s3Dict)
        s3Dict[attr] = reset

        try:
            sb.components.repository.s3Repo.S3Repo.validate(s3Dict)
        except SchemaError as error:
            self.assertEqual(error.code, "S3 '{attr}' must be a {expected}".format(attr=attr, expected=expected))

    def validate_error_artifactory(self, attr, reset, expected):
        artifactoryDict = copy.deepcopy(self.artifactoryDict)
        artifactoryDict[attr] = reset

        try:
            sb.components.repository.artifactoryRepo.ArtifactoryRepo.validate(artifactoryDict)
        except SchemaError as error:
            self.assertEqual(error.code, "Artifactory '{attr}' must be a{expected}".format(attr=attr, expected=expected))

    def validate_error_artifact(self, attr, reset, expected):
        artifactDict = copy.deepcopy(self.artifactDict)
        artifactDict[attr] = reset

        try:
            sb.components.repository.repository.Artifact.validate(artifactDict)
        except SchemaError as error:
            self.assertEqual(error.code, "Artifact '{attr}' must be a {expected}".format(attr=attr, expected=expected))

    def test_invalid(self):
        self.validate_error_s3('bucket', 123, 'String')
        self.validate_error_s3('region', 123, 'String')
        self.validate_error_s3('profile', 123, 'String')
        self.validate_error_artifactory('url', 123, ' String')
        self.validate_error_artifactory('repo', 123, ' String')
        self.validate_error_artifactory('path', 123, ' String')
        self.validate_error_artifactory('auth', 123, 'n Object')
        self.validate_error_artifact('name', 123, 'String')
        self.validate_error_artifact('file', 123, 'String')

if __name__ == '__main__':
    main()



"""Kerberos Component"""

from schema import Schema, And
from ..objects.component import Activation, Component

class Kerberos(Component):
    """
    Kerberos Class

    Provides the ability authentice with Kerberos HDFS based on user config
    """

    activation = Activation.CONFIG

    schema = Schema({
        'krbConf': And(str, error='Kerberos \'krbConf\' must be a String'),
        'keytab': And(str, error='Kerberos \'keytab\' must be a String'),
        'hdfsUser': And(str, error='Kerberos \'hdfsUser\' must be a String')
    }, ignore_extra_keys=True)

    krbConf = None
    keytab = None
    hdfsUser = None

    def __init__(self, krbConf=None, keytab=None, hdfsUser=None):
        """Initialize the kerberos auth details when constructing the component object"""

        self.krbConf = krbConf
        self.keytab = keytab
        self.hdfsUser = hdfsUser

    def appendDockerfile(self):
        """
        Dockerfile Hook

        Appends copy commands to the Dockerfile in order to add the needed Kerberos files to the
        correct location based on the config
        """

        dockerfile = ""
        if (self.krbConf is not None) and (self.keytab is not None):
            dockerfile += "COPY {krb} /etc/krb5.conf\n".format(krb=self.krbConf)
            dockerfile += "COPY {keytab} /krb/auth.keytab\n".format(keytab=self.keytab)
        return dockerfile

    def prependCommand(self, job, native):
        """
        CommandBuilder Hook

        If run inside of docker, execute the init.sh script with the username in order to
        initialize krb auth so that the user can access HDFS inside the container
        """

        return "/./krb/init.sh {user}".format(user=self.hdfsUser) if native is False else None



"""Repository Component"""

from schema import Schema, And, Optional, SchemaError
from .s3Repo import S3Repo
from .artifactoryRepo import ArtifactoryRepo
from ...objects.component import Activation, Component
from ...objects.skeleYaml import SkeleYaml

ERROR_NOT_COMPATIBLE = "No Compatible Version Found"
ERROR_ALREADY_PUSHED = "This artifact version already exists. Please bump the version or use the force parameter (-f) to overwrite the artifact."

class Artifact(SkeleYaml):
    """
    Artifact Class

    Object contained within a list of the artifactory configuration in order to specify the
    artifact files and artifact names
    """

    VERSIONED_NAME = "{prefix}{filename}_v{version}.{ext}"
    SINGULAR_NAME = "{prefix}{filename}.{ext}"

    schema = Schema({
        'name': And(str, error="Artifact 'name' must be a String"),
        'file': And(str, error="Artifact 'file' must be a String"),
        Optional('singular'): And(bool, error="Artifact 'singular' must be a Boolean"),
    }, ignore_extra_keys=True)

    name = None
    file = None
    singular = None

    def __init__(self, name, file, singular=False):
        """ Initialize the Artifact object with a name to be used in commands a file path """
        self.name = name
        self.file = file
        self.singular = singular

    def getVersionedName(self, version, prefix=None):
        """
        Constuct the full versioned filename for the artifact with the given version number
        or without the number if it is a 'singular' artifact instead of a versioned artifact
        """
        ext = self.file.split(".")[1]
        ver_name = None
        prefix = prefix if prefix is not None else ""
        if (self.singular):
            ver_name = self.SINGULAR_NAME.format(prefix=prefix, filename=self.name, ext=ext)
        else:
            ver_name = self.VERSIONED_NAME.format(prefix=prefix, filename=self.name, version=version, ext=ext)
        return ver_name


class Repository(Component):
    """
    Repository Component Class

    Provides the ability to push and pull artifacts that are defined in the skelebot config
    file to and from Artifactory or S3 based on the project version number
    """

    activation = Activation.CONFIG
    commands = ["push", "pull"]

    schema = Schema({
        Optional('artifacts'): And(list, error="Repository 'artifacts' must be a List"),
        Optional('s3'): And(dict, error="Repository 's3' must be an Object"),
        Optional('artifactory'): And(dict, error="Repository 'artifactory' must be an Object"),
    }, ignore_extra_keys=True)

    artifacts = None
    s3 = None
    artifactory = None

    @classmethod
    def load(cls, config):
        """Instantiate the Repository Class Object based on a config dict"""

        cls.validate(config)

        s3 = None
        artifactory = None
        if ("s3" in config):
            s3 = S3Repo.load(config["s3"])
        elif ("artifactory" in config):
            artifactory = ArtifactoryRepo.load(config["artifactory"])
        else:
            raise SchemaError(None, "Repository must contain 's3' or 'artifactory' config")

        artifactDicts = config["artifacts"]
        artifacts = []
        for artifact in artifactDicts:
            newArtifact = Artifact.load(artifact)
            artifacts.append(newArtifact)

        return cls(artifacts, s3, artifactory)

    def __init__(self, artifacts=None, s3=None, artifactory=None):
        """Instantiate the Repository Class Object based on the provided parameters"""

        self.artifacts = artifacts if artifacts is not None else []
        self.s3 = s3
        self.artifactory = artifactory

    def requiresUsername(self):
        return (self.artifactory is not None) and (self.artifactory.requiresUsername())

    def requiresToken(self):
        return (self.artifactory is not None) and (self.artifactory.requiresToken())

    def addParsers(self, subparsers):
        """
        SkeleParser Hook

        Adds the parsers for push and pull commands to the SkeleParser to push artifacts
        with the project version and pull artifacts with the provided version number
        """

        artifactNames = ["ALL"]
        for artifact in self.artifacts:
            artifactNames.append(artifact.name)

        repo = "Artifactory" if self.artifactory is not None else "S3"

        parser = subparsers.add_parser("push", help="Push an artifact to {}".format(repo))
        parser.add_argument("artifact", choices=artifactNames)
        parser.add_argument("-f", "--force", action='store_true', help="Force the push")
        parser.add_argument("-p", "--prefix", help="Prefix the artifact name or names with text")
        if (self.requiresUsername()):
            parser.add_argument("-u", "--user", help="Auth user for Artifactory")
        if (self.requiresToken()):
            parser.add_argument("-t", "--token", help="Auth password/token/apikey for Artifactory")

        parser = subparsers.add_parser("pull", help="Pull an artifact from {}".format(repo))
        parser.add_argument("artifact", choices=artifactNames)
        parser.add_argument("version", help="The version of the artifact to pull")
        parser.add_argument("-o", "--override", action='store_true',  help="Override the model in the existing directory")
        if (self.requiresUsername()):
            parser.add_argument("-u", "--user", help="Auth user for Artifactory")
        if (self.requiresToken()):
            parser.add_argument("-t", "--token", help="Auth password/token/apikey for Artifactory")

        return subparsers

    def execute(self, config, args, host=None):

        # Obtain the user and token if required
        user = None
        token = None
        if (self.requiresUsername()):
            user = args.user
        if (self.requiresToken()):
            token = args.token

        # Obtain the configured artifact repository
        artifactRepo = self.s3 if self.s3 is not None else self.artifactory

        # Obtain the artifact(s) that matches the provided name
        for artifact in self.artifacts:
            if ((args.artifact == artifact.name) | (args.artifact == "ALL")):

                # Push or Pull the Artifact
                if (args.job == "push"): # Push from Disk to Repo
                    print("Pushing version {version} of {artifact}".format(version=config.version, artifact=artifact.name))
                    artifactRepo.push(artifact, config.version, args.force, user, token, args.prefix)
                elif (args.job == "pull"): # Pull from Repo to Disk
                    print("Pulling version {version} of {artifact}".format(version=args.version, artifact=artifact.name))
                    artifactRepo.pull(artifact, args.version, config.version, args.override, user, token)



import argparse
import copy
import unittest
from unittest import mock

from schema import SchemaError

import skelebot as sb

class TestRegistry(unittest.TestCase):

    registry = {
        "host": "docker.io",
        "port": 88,
        "user": "skelebot",
        "aws": {
            "region": "us-east-1",
            "profile": "dev"
        }
    }

    def test_addParsers(self):
        registry = sb.components.registry.Registry(host="docker.io", port=88, user="skelebot")

        parser = argparse.ArgumentParser(formatter_class=argparse.RawTextHelpFormatter)
        subparsers = parser.add_subparsers(dest="job")
        subparsers = registry.addParsers(subparsers)

        self.assertNotEqual(subparsers.choices["publish"], None)

    @mock.patch('skelebot.components.registry.docker')
    def test_execute(self, mock_docker):
        mock_docker.build.return_value = 0

        config = sb.objects.config.Config()
        args = argparse.Namespace(tags=None, skip_build_global=False, verbose_global=False,
                                    omit_version=False, omit_latest=False)

        registry = sb.components.registry.Registry(host="docker.io", port=88, user="skelebot")
        registry.execute(config, args)

        mock_docker.login.assert_called_with(host="docker.io", docker_host=None, verbose=False)
        mock_docker.build.assert_called_with(config, host=None, verbose=False)
        mock_docker.push.assert_called_with(
            config, "docker.io", 88, "skelebot", tags=None, docker_host=None, verbose=False,
            omit_latest=False, omit_version=False
        )

    @mock.patch('skelebot.components.registry.docker')
    def test_execute_omit_latest(self, mock_docker):
        mock_docker.build.return_value = 0

        config = sb.objects.config.Config()
        args = argparse.Namespace(tags=None, skip_build_global=False, verbose_global=False,
                                    omit_version=False, omit_latest=True)

        registry = sb.components.registry.Registry(host="docker.io", port=88, user="skelebot")
        registry.execute(config, args)

        mock_docker.login.assert_called_with(host="docker.io", docker_host=None, verbose=False)
        mock_docker.build.assert_called_with(config, host=None, verbose=False)
        mock_docker.push.assert_called_with(
            config, "docker.io", 88, "skelebot", tags=None, docker_host=None, verbose=False,
            omit_latest=True, omit_version=False
        )

    @mock.patch('skelebot.components.registry.docker')
    def test_execute_omit_version(self, mock_docker):
        mock_docker.build.return_value = 0

        config = sb.objects.config.Config()
        args = argparse.Namespace(tags=None, skip_build_global=False, verbose_global=False,
                                    omit_version=True, omit_latest=False)

        registry = sb.components.registry.Registry(host="docker.io", port=88, user="skelebot")
        registry.execute(config, args)

        mock_docker.login.assert_called_with(host="docker.io", docker_host=None, verbose=False)
        mock_docker.build.assert_called_with(config, host=None, verbose=False)
        mock_docker.push.assert_called_with(
            config, "docker.io", 88, "skelebot", tags=None, docker_host=None, verbose=False,
            omit_latest=False, omit_version=True
        )

    @mock.patch('skelebot.components.registry.docker')
    def test_execute_skip_build(self, mock_docker):
        mock_docker.build.return_value = 0

        config = sb.objects.config.Config()
        args = argparse.Namespace(tags=None, skip_build_global=True, verbose_global=False,
                                    omit_version=False, omit_latest=False)

        registry = sb.components.registry.Registry(host="docker.io", port=88, user="skelebot")
        registry.execute(config, args)

        mock_docker.login.assert_called_with(host="docker.io", docker_host=None, verbose=False)
        mock_docker.push.assert_called_with(
            config, "docker.io", 88, "skelebot", tags=None, docker_host=None, verbose=False,
            omit_latest=False, omit_version=False
        )

    @mock.patch('skelebot.components.registry.docker')
    def test_execute_host(self, mock_docker):
        mock_docker.build.return_value = 0

        config = sb.objects.config.Config()
        args = argparse.Namespace(tags=None, skip_build_global=False, verbose_global=False,
                                    omit_version=False, omit_latest=False)

        registry = sb.components.registry.Registry(host="docker.io", port=88, user="skelebot")
        registry.execute(config, args, host="host1")

        mock_docker.login.assert_called_with(host="docker.io", docker_host="host1", verbose=False)
        mock_docker.build.assert_called_with(config, host="host1", verbose=False)
        mock_docker.push.assert_called_with(
            config, "docker.io", 88, "skelebot", tags=None, docker_host="host1", verbose=False,
            omit_latest=False, omit_version=False
        )

    @mock.patch('skelebot.components.registry.docker')
    def test_execute_aws(self, mock_docker):
        mock_docker.build.return_value = 0

        config = sb.objects.config.Config()
        args = argparse.Namespace(tags=None, skip_build_global=False, verbose_global=False,
                                    omit_version=False, omit_latest=False)

        aws = sb.components.registry.Aws(region="us-east-1", profile="dev")
        registry = sb.components.registry.Registry(host="docker.io", port=88, user="skelebot", aws=aws)
        registry.execute(config, args)

        mock_docker.loginAWS.assert_called_with(
            "docker.io", "us-east-1", "dev", docker_host=None, verbose=False
        )
        mock_docker.build.assert_called_with(config, host=None, verbose=False)
        mock_docker.push.assert_called_with(
            config, "docker.io", 88, "skelebot", tags=None, docker_host=None, verbose=False,
            omit_latest=False, omit_version=False
        )

    @mock.patch('skelebot.components.registry.docker')
    def test_execute_aws_host(self, mock_docker):
        mock_docker.build.return_value = 0

        config = sb.objects.config.Config()
        args = argparse.Namespace(tags=None, skip_build_global=False, verbose_global=False,
                                    omit_version=False, omit_latest=False)

        aws = sb.components.registry.Aws(region="us-east-1", profile="dev")
        registry = sb.components.registry.Registry(host="docker.io", port=88, user="skelebot", aws=aws)
        registry.execute(config, args, host="host1")

        mock_docker.loginAWS.assert_called_with(
            "docker.io", "us-east-1", "dev", docker_host='host1', verbose=False
        )
        mock_docker.build.assert_called_with(config, host="host1", verbose=False)
        mock_docker.push.assert_called_with(
            config, "docker.io", 88, "skelebot", tags=None, docker_host="host1", verbose=False,
            omit_latest=False, omit_version=False
        )

    @mock.patch('skelebot.components.registry.docker')
    def test_execute_tags(self, mock_docker):
        mock_docker.build.return_value = 0

        config = sb.objects.config.Config()
        args = argparse.Namespace(tags=["test", "dev", "stage"], skip_build_global=False, verbose_global=False,
                                    omit_version=False, omit_latest=False)

        registry = sb.components.registry.Registry(host="docker.io", port=88, user="skelebot")
        registry.execute(config, args)

        mock_docker.login.assert_called_with(host="docker.io", docker_host=None, verbose=False)
        mock_docker.build.assert_called_with(config, host=None, verbose=False)
        mock_docker.push.assert_called_with(
            config, "docker.io", 88, "skelebot", tags=['test', 'dev', 'stage'],
            docker_host=None, verbose=False,
            omit_latest=False, omit_version=False
        )

    def test_validate_valid(self):
        try:
            sb.components.registry.Registry.validate(self.registry)
        except:
            self.fail("Validation Raised Exception Unexpectedly")

    def validate_error(self, attr, reset, expected):
        registry = copy.deepcopy(self.registry)
        registry[attr] = reset

        try:
            sb.components.registry.Registry.validate(registry)
        except SchemaError as error:
            self.assertEqual(error.code, "Registry '{attr}' must be a{expected}".format(attr=attr, expected=expected))

    def validate_error_aws(self, attr, reset, expected):
        registry = copy.deepcopy(self.registry)
        registry['aws'][attr] = reset

        try:
            sb.components.registry.Registry.validate(registry)
        except SchemaError as error:
            self.assertEqual(error.code, "Registry AWS '{attr}' must be a{expected}".format(attr=attr, expected=expected))


    def test_invalid(self):
        self.validate_error('host', 123, ' String')
        self.validate_error('port', "abc", 'n Integer')
        self.validate_error('user', 123, ' String')
        self.validate_error('region', 123, ' String')
        self.validate_error('profile', 123, ' String')

        try:
            sb.components.registry.Registry.validate({"host": "abc.com", "aws": {}})
        except SchemaError as error:
            self.assertEqual(error.code, "Registry AWS 'region' is required")

if __name__ == '__main__':
    unittest.main()



import unittest

import skelebot as sb

class TestSemver(unittest.TestCase):

    def test_representation(self):
        version = sb.objects.semver.Semver('1.2.3')
        self.assertEqual(version.major, 1)
        self.assertEqual(version.minor, 2)
        self.assertEqual(version.patch, 3)
        self.assertEqual(str(version), '1.2.3')

    def test_lt(self):
        a = sb.objects.semver.Semver('0.1.2')
        b = sb.objects.semver.Semver('1.1.2')
        c = sb.objects.semver.Semver('1.2.3')
        d = sb.objects.semver.Semver('1.2.10')
        self.assertTrue(a < b)
        self.assertTrue(b < c)
        self.assertTrue(c < d)

    def test_eq(self):
        a = sb.objects.semver.Semver('1.2.3')
        b = sb.objects.semver.Semver('1.2.3')
        c = sb.objects.semver.Semver('1.2.4')
        d = sb.objects.semver.Semver('1.4.3')
        e = sb.objects.semver.Semver('2.2.3')
        self.assertEqual(a, b)
        # Different patch
        self.assertNotEqual(a, c)
        # Different minor
        self.assertNotEqual(a, d)
        # Different major
        self.assertNotEqual(a, e)

    def test_le(self):
        a = sb.objects.semver.Semver('1.2.3')
        b = sb.objects.semver.Semver('1.2.3')
        c = sb.objects.semver.Semver('2.2.3')
        self.assertTrue(a <= b)
        self.assertTrue(a <= c)

    def test_isBackwardCompatible(self):
        a = sb.objects.semver.Semver('1.2.3')
        b = sb.objects.semver.Semver('1.2.10')
        c = sb.objects.semver.Semver('2.2.3')
        self.assertTrue(b.isBackwardCompatible(a))
        self.assertFalse(a.isBackwardCompatible(b))
        self.assertFalse(a.isBackwardCompatible(c))
        self.assertFalse(b.isBackwardCompatible(c))

if __name__ == '__main__':
    unittest.main()



import argparse
import unittest
from unittest import mock

from colorama import Fore, Style

import skelebot as sb

class TestExecutor(unittest.TestCase):

    @mock.patch('skelebot.systems.execution.executor.print')
    @mock.patch('skelebot.systems.parsing.skeleParser')
    @mock.patch('skelebot.systems.execution.executor.VERSION', '6.6.6')
    def test_execute_version(self, mock_skeleParser, mock_print):
        config = sb.objects.config.Config()
        args = argparse.Namespace(job=None, version_global=True)
        mock_skeleParser.parseArgs.return_value = args

        sb.systems.execution.executor.execute(config, mock_skeleParser)

        mock_print.assert_called_with("Skelebot v6.6.6")

    @mock.patch('skelebot.systems.execution.executor.print')
    @mock.patch('skelebot.systems.parsing.skeleParser')
    def test_execute_contact(self, mock_skeleParser, mock_print):
        config = sb.objects.config.Config(contact="me@my-mail.com")
        args = argparse.Namespace(job=None, contact_global=True)
        mock_skeleParser.parseArgs.return_value = args

        sb.systems.execution.executor.execute(config, mock_skeleParser)

        mock_print.assert_called_with("me@my-mail.com")

    @mock.patch('skelebot.systems.parsing.skeleParser')
    def test_execute_help(self, mock_skeleParser):
        config = sb.objects.config.Config()
        args = argparse.Namespace(job=None)
        mock_skeleParser.parseArgs.return_value = args

        sb.systems.execution.executor.execute(config, mock_skeleParser)

        mock_skeleParser.showHelp.assert_called_once()

    @mock.patch('skelebot.systems.execution.executor.Scaffolder')
    @mock.patch('skelebot.systems.parsing.skeleParser')
    def test_execute_scaffold(self, mock_skeleParser, mock_scaffolder):
        config = sb.objects.config.Config()
        args = argparse.Namespace(job="scaffold", existing=False, template=None)
        mock_skeleParser.parseArgs.return_value = args
        mock_scaffold = mock.MagicMock()
        mock_scaffolder.return_value = mock_scaffold

        sb.systems.execution.executor.execute(config, mock_skeleParser)

        mock_scaffolder.assert_called_once_with(existing=False)
        mock_scaffold.scaffold.assert_called_once()

    @mock.patch('skelebot.systems.execution.executor.runDocker')
    @mock.patch('skelebot.systems.parsing.skeleParser')
    def test_execute_job_skip(self, mock_skeleParser, mock_run):
        job = sb.objects.job.Job(name="test", source="test.py")
        config = sb.objects.config.Config(jobs=[job])
        args = argparse.Namespace(
            job="test", native_global=False, skip_build_global=True,
            verbose_global=False
        )
        mock_skeleParser.parseArgs.return_value = args
        mock_run.return_value = 0

        sb.systems.execution.executor.execute(config, mock_skeleParser)

        mock_run.assert_called_once_with(
            config, "python -u test.py", "i", [], [], "test", host=None, verbose=False
        )

    @mock.patch('skelebot.systems.execution.executor.buildDocker')
    @mock.patch('skelebot.systems.execution.executor.runDocker')
    @mock.patch('skelebot.systems.parsing.skeleParser')
    def test_execute_job(self, mock_skeleParser, mock_run, mock_build):
        job = sb.objects.job.Job(name="test", source="test.py")
        config = sb.objects.config.Config(jobs=[job])
        args = argparse.Namespace(
            job="test", native_global=False, skip_build_global=False, verbose_global=False
        )
        mock_skeleParser.parseArgs.return_value = args
        mock_run.return_value = 0

        sb.systems.execution.executor.execute(config, mock_skeleParser)

        mock_build.assert_called_once_with(config, host=None, verbose=False)
        mock_run.assert_called_once_with(
            config, "python -u test.py", "i", [], [], "test", host=None, verbose=False
        )

    @mock.patch('skelebot.systems.execution.executor.buildDocker')
    @mock.patch('skelebot.systems.execution.executor.runDocker')
    @mock.patch('skelebot.systems.parsing.skeleParser')
    def test_execute_job_host_global(self, mock_skeleParser, mock_run, mock_build):
        job = sb.objects.job.Job(name="test", source="test.py")
        config = sb.objects.config.Config(jobs=[job], host="host1")
        args = argparse.Namespace(
            job="test", native_global=False, skip_build_global=False, verbose_global=False
        )
        mock_skeleParser.parseArgs.return_value = args
        mock_run.return_value = 0

        sb.systems.execution.executor.execute(config, mock_skeleParser)

        mock_build.assert_called_once_with(config, host="host1", verbose=False)
        mock_run.assert_called_once_with(
            config, "python -u test.py", "i", [], [], "test", host="host1", verbose=False
        )

    @mock.patch('skelebot.systems.execution.executor.buildDocker')
    @mock.patch('skelebot.systems.execution.executor.runDocker')
    @mock.patch('skelebot.systems.parsing.skeleParser')
    def test_execute_job_host_job(self, mock_skeleParser, mock_run, mock_build):
        job = sb.objects.job.Job(name="test", source="test.py", host="host2")
        config = sb.objects.config.Config(jobs=[job], host="host1")
        args = argparse.Namespace(
            job="test", native_global=False, skip_build_global=False, verbose_global=False
        )
        mock_skeleParser.parseArgs.return_value = args
        mock_run.return_value = 0

        sb.systems.execution.executor.execute(config, mock_skeleParser)

        mock_build.assert_called_once_with(config, host="host2", verbose=False)
        mock_run.assert_called_once_with(
            config, "python -u test.py", "i", [], [], "test", host="host2", verbose=False
        )

    @mock.patch('skelebot.systems.execution.executor.buildDocker')
    @mock.patch('skelebot.systems.execution.executor.runDocker')
    @mock.patch('skelebot.systems.parsing.skeleParser')
    def test_execute_job_host_param(self, mock_skeleParser, mock_run, mock_build):
        job = sb.objects.job.Job(name="test", source="test.py", host="host2")
        config = sb.objects.config.Config(jobs=[job], host="host1")
        args = argparse.Namespace(
            job="test", native_global=False, skip_build_global=False, host="host3",
            verbose_global=False
        )
        mock_skeleParser.parseArgs.return_value = args
        mock_run.return_value = 0

        sb.systems.execution.executor.execute(config, mock_skeleParser)

        mock_build.assert_called_once_with(config, host="host3", verbose=False)
        mock_run.assert_called_once_with(
            config, "python -u test.py", "i", [], [],
            "test", host="host3", verbose=False
        )

    @mock.patch('skelebot.systems.execution.executor.buildDocker')
    @mock.patch('skelebot.systems.execution.executor.runDocker')
    @mock.patch('skelebot.systems.parsing.skeleParser')
    def test_execute_job_ports(self, mock_skeleParser, mock_run, mock_build):
        job = sb.objects.job.Job(name="test", source="test.py", ports=["10:10", "20:20"])
        config = sb.objects.config.Config(jobs=[job], ports=["30:30", "10:10"])
        args = argparse.Namespace(
            job="test", native_global=False, skip_build_global=False, verbose_global=False
        )
        mock_skeleParser.parseArgs.return_value = args
        mock_run.return_value = 0

        sb.systems.execution.executor.execute(config, mock_skeleParser)

        mock_build.assert_called_once_with(config, host=None, verbose=False)
        mock_run.assert_called_once_with(
            config, "python -u test.py", "i", ["10:10", "20:20", "30:30"], [],
            "test", host=None, verbose=False
        )

    @mock.patch('skelebot.systems.execution.executor.call')
    @mock.patch('skelebot.systems.parsing.skeleParser')
    def test_execute_job_native(self, mock_skeleParser, mock_call):
        job = sb.objects.job.Job(name="test", source="test.py")
        config = sb.objects.config.Config(jobs=[job])
        args = argparse.Namespace(job="test", native_global=True, verbose_global=False)
        mock_skeleParser.parseArgs.return_value = args
        mock_call.return_value = 0

        sb.systems.execution.executor.execute(config, mock_skeleParser)

        mock_call.assert_called_once_with("python -u test.py", shell=True)

    @mock.patch('skelebot.systems.execution.executor.print')
    @mock.patch('skelebot.systems.execution.executor.call')
    @mock.patch('skelebot.systems.parsing.skeleParser')
    def test_execute_job_native_verbose(self, mock_skeleParser, mock_call, mock_print):
        job = sb.objects.job.Job(name="test", source="test.py")
        config = sb.objects.config.Config(jobs=[job])
        args = argparse.Namespace(job="test", native_global=True, verbose_global=True)
        mock_skeleParser.parseArgs.return_value = args
        mock_call.return_value = 0

        sb.systems.execution.executor.execute(config, mock_skeleParser)

        mock_print.assert_called_once_with(
            Fore.GREEN + "INFO" + Style.RESET_ALL + " | python -u test.py"
        )
        mock_call.assert_called_once_with("python -u test.py", shell=True)

    @mock.patch('skelebot.systems.execution.executor.call')
    @mock.patch('skelebot.systems.parsing.skeleParser')
    def test_execute_job_native_always(self, mock_skeleParser, mock_call):
        job = sb.objects.job.Job(name="test", source="test.py", native="always")
        config = sb.objects.config.Config(jobs=[job])
        args = argparse.Namespace(job="test", native_global=False, verbose_global=False)
        mock_skeleParser.parseArgs.return_value = args
        mock_call.return_value = 0

        sb.systems.execution.executor.execute(config, mock_skeleParser)

        mock_call.assert_called_once_with("python -u test.py", shell=True)

    @mock.patch('skelebot.systems.execution.executor.buildDocker')
    @mock.patch('skelebot.systems.execution.executor.runDocker')
    @mock.patch('skelebot.systems.parsing.skeleParser')
    def test_execute_job_native_never(self, mock_skeleParser, mock_run, mock_build):
        job = sb.objects.job.Job(name="test", source="test.py", native="never")
        config = sb.objects.config.Config(jobs=[job])
        args = argparse.Namespace(
            job="test", native_global=True, skip_build_global=False, verbose_global=False
        )
        mock_skeleParser.parseArgs.return_value = args
        mock_run.return_value = 0

        sb.systems.execution.executor.execute(config, mock_skeleParser)

        mock_build.assert_called_once_with(config, host=None, verbose=False)
        mock_run.assert_called_once_with(
            config, "python -u test.py", "i", [], [], "test", host=None, verbose=False
        )

    @mock.patch('skelebot.systems.parsing.skeleParser')
    def test_execute_component(self, mock_skeleParser):
        mock_component = mock.MagicMock()
        mock_component.commands = ["test"]
        config = sb.objects.config.Config(components=[mock_component])
        args = argparse.Namespace(job="test", verbose_global=False)
        mock_skeleParser.parseArgs.return_value = args

        sb.systems.execution.executor.execute(config, mock_skeleParser)

        mock_component.execute.assert_called_once_with(config, args, host=None)

    @mock.patch('skelebot.systems.execution.executor.runDocker')
    @mock.patch('skelebot.systems.parsing.skeleParser')
    def test_execute_chain(self, mock_skeleParser, mock_run):
        job = sb.objects.job.Job(name="test", source="test.py")
        config = sb.objects.config.Config(jobs=[job])
        args = argparse.Namespace(
            job="test", native_global=False, skip_build_global=True, verbose_global=False
        )
        mock_skeleParser.parseArgs.return_value = args
        mock_run.return_value = 0

        sb.systems.execution.executor.execute(config, mock_skeleParser, ["test", "+", "test"])

        test_call = mock.call(config, "python -u test.py", "i", [], [], "test", host=None, verbose=False)
        mock_run.assert_has_calls([test_call, test_call])

    @mock.patch('skelebot.systems.execution.executor.runDocker')
    @mock.patch('skelebot.systems.parsing.skeleParser')
    def test_execute_chain_fail(self, mock_skeleParser, mock_run):
        job = sb.objects.job.Job(name="test", source="test.py")
        config = sb.objects.config.Config(jobs=[job])
        args = argparse.Namespace(
            job="test", native_global=False, skip_build_global=True, verbose_global=False
        )
        mock_skeleParser.parseArgs.return_value = args
        mock_run.return_value = 1
        try:
            sb.systems.execution.executor.execute(config, mock_skeleParser, ["test", "+", "test"])
            self.fail('exception expected')
        except SystemExit:
            mock_run.assert_called_once_with(
                config, "python -u test.py", "i", [], [], "test", host=None, verbose=False
            )

if __name__ == '__main__':
    unittest.main()



import copy
import unittest

from schema import SchemaError

import skelebot as sb

class TestArgValidate(unittest.TestCase):

    arg = {
        'name': 'test',
        'help': 'test',
        'choices': [1, 2]
    }

    def validate_error(self, attr, reset, expected):
        arg = copy.deepcopy(self.arg)
        arg[attr] = reset

        try:
            sb.objects.arg.Arg.validate(arg)
        except SchemaError as error:
            self.assertEqual(error.code, "Arg '{attr}' must be a {expected}".format(attr=attr, expected=expected))

    def test_valid(self):

        try:
            sb.objects.arg.Arg.validate(self.arg)
        except:
            self.fail("Validation Raised Exception Unexpectedly")

    def test_invalid_mising(self):
        arg = copy.deepcopy(self.arg)
        del arg['name']

        try:
            sb.objects.arg.Arg.validate(arg)
        except SchemaError as error:
            self.assertEqual(error.code, "Missing key: 'name'")

    def test_invalid(self):
        self.validate_error('name', 123, 'String')
        self.validate_error('help', 123, 'String')
        self.validate_error('choices', 123, 'List')

if __name__ == '__main__':
    unittest.main()



import shutil
import os
import json
from getpass import getpass
import artifactory
import boto3
from schema import Schema, And, Optional, Or
from .artifactRepo import ArtifactRepo
from ...objects.skeleYaml import SkeleYaml
from ...objects.semver import Semver

ARTIFACT_PATH = "{url}/{repo}/{path}/{file}"
AUTH_TYPES = {"user_pass", "token", "apikey"}

class Auth(SkeleYaml):
    """
    Artifactory Specific Authentication Details

    Provides the fields necessary for authenticating with Artifactory.
    """

    schema = Schema({
        Optional('authType'): And(str, Or(*AUTH_TYPES), error="Artifactory Auth 'authType' must be one of: " + ', '.join(AUTH_TYPES)),
        Optional('aws'): And(bool, error="Artifactory Auth 'aws' must be a Boolean"),
        Optional('awsSecret'): And(str, error="Artifactory Auth 'awsSecret' must be a String"),
        Optional('awsProfile'): And(str, error="Artifactory Auth 'awsProfile' must be a String"),
        Optional('awsRegion'): And(str, error="Artifactory Auth 'awsRegion' must be a String"),
    }, ignore_extra_keys=True)

    authType = None
    aws = None
    awsSecret = None
    awsProfile = None
    awsRegion = None

    def __init__(self, authType="user_pass", aws=False, awsSecret="Artifactory", awsProfile=None,
                 awsRegion="us-east-1"):
        """Instantiate the Auth Class Object based on the provided parameters"""
        self.authType = authType
        self.aws = aws
        self.awsSecret = awsSecret
        self.awsProfile = awsProfile
        self.awsRegion = awsRegion

    def loadAwsCredentials(self):
        """Load authentication credentials from AWS Secrets Manager."""
        session = boto3.Session(profile_name=self.awsProfile)
        client = session.client(service_name="secretsmanager", region_name=self.awsRegion)
        return json.loads(client.get_secret_value(SecretId=self.awsSecret)["SecretString"])

    def getUsername(self):
        """ Prompt for an Artifactory username """
        return input("Please provide a valid Artifactory user: ")

    def getToken(self):
        """ Prompt for an Artifactory password/token/apikey """
        cred = "password" if self.authType == "user_pass" else self.authType
        return getpass(f"Please provide a valid Artifactory {cred}: ")

    def getCredentials(self, user=None, token=None):
        """Get authentication credentials either from user input or AWS."""
        if self.aws:
            aws_creds = self.loadAwsCredentials()
            user = aws_creds.get('user', None)
            token = aws_creds.get('token', None)
        else:
            # Obtain any missing credentials
            if (user is None) and (self.authType == "user_pass"):
                user = self.getUsername()
            if (token is None):
                token = self.getToken()

        creds = {}
        if self.authType == "user_pass":
            creds = {"auth": (user, token)}
        elif self.authType == "token":
            creds = {"token": token}
        elif self.authType == "apikey":
            creds = {"apikey": token}

        return creds


class ArtifactoryRepo(ArtifactRepo):
    """
    Artifactory Repository Class

    Contains details and functions for pushing and pulling artifacts from Artifactory
    """

    schema = Schema({
        'url': And(str, error="Artifactory 'url' must be a String"),
        'repo': And(str, error="Artifactory 'repo' must be a String"),
        'path': And(str, error="Artifactory 'path' must be a String"),
        Optional('auth'): And(dict, error="Artifactory 'auth' must be an Object"),
    }, ignore_extra_keys=True)

    url = None
    repo = None
    path = None
    auth = None

    @classmethod
    def load(cls, config):
        """Instantiate the Artifactory Auth Class Object if it is present"""

        cls.validate(config)
        auth = Auth.load(config.get("auth", {}))

        return cls(config.get("url"), config.get("repo"), config.get("path"), auth)

    def __init__(self, url, repo, path, auth=None):
        """ Initialize the values for an Artifactory connection and set the global config """
        self.url = url
        self.repo = repo
        self.path = path
        self.auth = auth or Auth()

    def requiresUsername(self):
        """Check if authentication requires username from user input."""
        return (not self.auth.aws) and (self.auth.authType == "user_pass")

    def requiresToken(self):
        """Check if authentication requires password/token/apikey from user input."""
        return (not self.auth.aws)

    def getLatestCompatibleVersion(self, artifact, currentVersion, creds):
        """ Search for the latest compatible version for the provided version number """

        print("Searching for Latest Compatible Artifact")
        version = None
        currentSemver = Semver(currentVersion)

        listUrl = "{url}/{repo}/{path}/".format(url=self.url, repo=self.repo, path=self.path)
        ext = artifact.file.split(".")[1]
        path = artifactory.ArtifactoryPath(listUrl, **creds)

        # Iterate over all artifacts in the ArtifactoryPath (because path.glob was throwing exceptions on Linux systems)
        if (path.exists()):
            for pathArtifact in path: # Only look at artifacts with the same filename and major version
                modelPrefix = "{filename}_v{major}".format(filename=artifact.name, major=currentSemver.major)
                if modelPrefix in str(pathArtifact):
                    artifactSemver = Semver(str(pathArtifact).split("_v")[1].split(ext)[0])
                    if (currentSemver.isBackwardCompatible(artifactSemver)) and ((version is None) or (version < artifactSemver)):
                        version = artifactSemver # Identify the latest compatible version

        # Raise an error if no compatible version is found
        if (version is None):
            raise RuntimeError(self.ERROR_NOT_COMPATIBLE)

        return version

    def push(self, artifact, version, force=False, user=None, token=None, prefix=None):
        """ Push the artifact with the given version number to Artifactory """

        creds = self.auth.getCredentials(user=user, token=token)

        artifactName = artifact.getVersionedName(version, prefix=prefix)
        url = ARTIFACT_PATH.format(url=self.url, repo=self.repo, path=self.path, file=artifactName)
        path = artifactory.ArtifactoryPath(url, **creds)

        # Error and exit if artifact already exists and we are not forcing an override
        if (not force) and (path.exists()):
            raise RuntimeError(self.ERROR_ALREADY_PUSHED)

        # Push the artifact to the path in Artifactory
        print("Deploying {file} to {url}".format(file=artifactName, url=url))
        shutil.copyfile(artifact.file, artifactName)
        try:
            path.deploy_file(artifactName)
            os.remove(artifactName)
        except:
            os.remove(artifactName)
            raise

    def pull(self, artifact, version, currentVersion=None, override=False, user=None, token=None):
        """ Pull the artifact with the given version number (or LATEST) from Artifactory """

        creds = self.auth.getCredentials(user=user, token=token)

        # Find Latest Compatible Version or Use Provided Version Number if the artifact is not 'singular'
        version = version if (version != "LATEST" or artifact.singular == True) else self.getLatestCompatibleVersion(artifact, currentVersion, creds)

        # Construct the Artifactory URL for the specified Artifact
        artifactName = artifact.getVersionedName(version)
        url = ARTIFACT_PATH.format(url=self.url, repo=self.repo, path=self.path, file=artifactName)
        path = artifactory.ArtifactoryPath(url, **creds)

        # Download Artifact from Artifactory and save it to the disk
        if (path.exists()):
            print("Pulling {file} from {url}".format(file=artifactName, url=url))
            with path.open() as fd:
                dest = artifact.file if (override) else artifactName
                with open(dest, "wb") as out:
                    out.write(fd.read())
        else:
            print("Artifact Not Found: {url}".format(url=url))



from unittest import TestCase, main, mock
import skelebot as sb

class TestAuth(TestCase):

    @mock.patch('boto3.Session')
    def test_load_aws_credentials(self, mock_boto3_session):
        mock_client = mock.Mock()
        mock_session = mock.Mock()
        mock_boto3_session.return_value = mock_session
        mock_session.client.return_value = mock_client
        mock_client.get_secret_value.return_value = {"SecretString": '{"a": 1}'}

        auth = sb.components.repository.artifactoryRepo.Auth(
            aws=True, awsSecret="foo", awsProfile="bar", awsRegion="baz"
        )
        actual_creds = auth.loadAwsCredentials()

        assert actual_creds == {"a": 1}
        mock_client.get_secret_value.assert_called_with(SecretId="foo")
        mock_session.client.assert_called_with(service_name="secretsmanager", region_name="baz")
        mock_boto3_session.assert_called_with(profile_name="bar")

    @mock.patch('boto3.Session')
    def test_load_aws_default_credentials(self, mock_boto3_session):
        mock_client = mock.Mock()
        mock_session = mock.Mock()
        mock_boto3_session.return_value = mock_session
        mock_session.client.return_value = mock_client
        mock_client.get_secret_value.return_value = {"SecretString": '{"a": 1}'}

        auth = sb.components.repository.artifactoryRepo.Auth(aws=True)
        actual_creds = auth.loadAwsCredentials()

        assert actual_creds == {"a": 1}
        mock_client.get_secret_value.assert_called_with(SecretId="Artifactory")
        mock_session.client.assert_called_with(service_name="secretsmanager", region_name="us-east-1")
        mock_boto3_session.assert_called_with(profile_name=None)

    @mock.patch('boto3.Session')
    def test_get_credentials_aws_user_pass(self, mock_boto3_session):
        mock_client = mock.Mock()
        mock_session = mock.Mock()
        mock_boto3_session.return_value = mock_session
        mock_session.client.return_value = mock_client
        mock_client.get_secret_value.return_value = {
            "SecretString": '{"user": "abc", "token": "def"}'
        }

        auth = sb.components.repository.artifactoryRepo.Auth(
            authType="user_pass", aws=True, awsSecret="foo", awsProfile="bar", awsRegion="baz"
        )

        actual_creds = auth.getCredentials()
        assert actual_creds == {"auth": ("abc", "def")}
        mock_client.get_secret_value.assert_called_with(SecretId="foo")
        mock_session.client.assert_called_with(service_name="secretsmanager", region_name="baz")
        mock_boto3_session.assert_called_with(profile_name="bar")

    @mock.patch('boto3.Session')
    def test_get_credentials_aws_token(self, mock_boto3_session):
        mock_client = mock.Mock()
        mock_session = mock.Mock()
        mock_boto3_session.return_value = mock_session
        mock_session.client.return_value = mock_client
        mock_client.get_secret_value.return_value = {
            "SecretString": '{"token": "def"}'
        }

        auth = sb.components.repository.artifactoryRepo.Auth(
            authType="token", aws=True, awsSecret="foo", awsProfile="bar", awsRegion="baz"
        )

        actual_creds = auth.getCredentials()
        assert actual_creds == {"token": "def"}
        mock_client.get_secret_value.assert_called_with(SecretId="foo")
        mock_session.client.assert_called_with(service_name="secretsmanager", region_name="baz")
        mock_boto3_session.assert_called_with(profile_name="bar")

    @mock.patch('skelebot.components.repository.artifactoryRepo.getpass')
    @mock.patch('skelebot.components.repository.artifactoryRepo.input')
    def test_get_credentials_input_user_pass(self, mock_input, mock_pass):
        mock_input.return_value = "abc"
        mock_pass.return_value = "def"

        auth = sb.components.repository.artifactoryRepo.Auth(authType="user_pass")
        actual_creds = auth.getCredentials()

        assert actual_creds == {"auth": ("abc", "def")}
        mock_input.assert_called_with("Please provide a valid Artifactory user: ")
        mock_pass.assert_called_with("Please provide a valid Artifactory password: ")

    @mock.patch('skelebot.components.repository.artifactoryRepo.getpass')
    @mock.patch('skelebot.components.repository.artifactoryRepo.input')
    def test_get_credentials_input_token(self, mock_input, mock_pass):
        mock_pass.return_value = "abc"

        auth = sb.components.repository.artifactoryRepo.Auth(authType="token")
        actual_creds = auth.getCredentials()

        assert actual_creds == {"token": "abc"}
        mock_input.asser_not_called()
        mock_pass.assert_called_with("Please provide a valid Artifactory token: ")

    @mock.patch('skelebot.components.repository.artifactoryRepo.getpass')
    @mock.patch('skelebot.components.repository.artifactoryRepo.input')
    def test_get_credentials_input_apikey(self, mock_input, mock_pass):
        mock_pass.return_value = "abc"

        auth = sb.components.repository.artifactoryRepo.Auth(authType="apikey")
        actual_creds = auth.getCredentials()

        assert actual_creds == {"apikey": "abc"}
        mock_input.asser_not_called()
        mock_pass.assert_called_with("Please provide a valid Artifactory apikey: ")

if __name__ == '__main__':
    main()



import copy
import unittest
from unittest import mock

from colorama import Fore, Style

from schema import SchemaError

import skelebot as sb

class TestConfigValidate(unittest.TestCase):

    config = {
        'name': 'test',
        'env': 'test',
        'description': 'test',
        'maintainer': 'test',
        'contact': 'test',
        'baseImage': 'test',
        'primaryJob': 'test',
        'host': 'test',
        'ephemeral': True,
        'dependencies': [1, 2],
        'ignores': [1, 2],
        'jobs': [1, 2],
        'ports': [1, 2],
        'components': {},
        'params': [1, 2],
        'commands': [],
        'pythonVersion': '3.9',
        'gpu': True
    }

    def validate_error(self, attr, reset, expected):
        config = copy.deepcopy(self.config)
        config[attr] = reset

        try:
            sb.objects.config.Config.validate(config)
        except SchemaError as error:
            self.assertEqual(error.code, "'{attr}' must be {expected}".format(attr=attr, expected=expected))

    def test_valid(self):

        try:
            sb.objects.config.Config.validate(self.config)
        except:
            self.fail("Validation Raised Exception Unexpectedly")

    @mock.patch.object(sb.objects.config, 'DEPRECATED_VERSIONS', ['3.6'])
    @mock.patch('skelebot.objects.config.print')
    def test_valid_deprecated(self, mock_print):
        _ = sb.objects.config.Config(pythonVersion='3.6')
        mock_print.assert_called_once_with(
            Fore.YELLOW + "WARN" + Style.RESET_ALL
            + " | The support for Python version 3.6 has been deprecated as of v1.37.0."
            + " Please use a higher version."
        )

    def test_invalid_mising(self):
        config = copy.deepcopy(self.config)
        del config['name']

        try:
            sb.objects.config.Config.validate(config)
        except SchemaError as error:
            self.assertEqual(error.code, "Missing key: 'name'")

    def test_invalid(self):
        self.validate_error('name', 123, 'a String')
        self.validate_error('env', 123, 'a String')
        self.validate_error('description', 123, 'a String')
        self.validate_error('maintainer', 123, 'a String')
        self.validate_error('contact', 123, 'a String')
        self.validate_error('baseImage', 123, 'a String')
        self.validate_error('primaryJob', 123, 'a String')
        self.validate_error('primaryExe', 123, 'CMD or ENTRYPOINT')
        self.validate_error('primaryExe', 'bad', 'CMD or ENTRYPOINT')
        self.validate_error('ephemeral', 123, 'a Boolean')
        self.validate_error('dependencies', 123, 'a Dict or List')
        self.validate_error('ignores', 123, 'a List')
        self.validate_error('jobs', 123, 'a List')
        self.validate_error('ports', 123, 'a List')
        self.validate_error('components', 123, 'a Dictionary')
        self.validate_error('params', 123, 'a List')
        self.validate_error('commands', 123, 'a List')
        self.validate_error('host', 123, 'a String')
        self.validate_error('pythonVersion', 123, 'one of:' + ', '.join(sb.common.PYTHON_VERSIONS))
        self.validate_error('gpu', 123, 'a Boolean')

if __name__ == '__main__':
    unittest.main()



"""Docker Command Builder"""

EP_COMMAND = " {command} {image} {parameters}"
EX_COMMAND = " {image} /bin/bash -c \"{command}\""

class SaveCommandBuilder():
    cmd = None
    image = None

    def __init__(self, cmd, image):
        self.image = image
        self.cmd = cmd + " save"

    def set_output(self, filename):
        self.cmd += " -o {}".format(filename)
        return self

    def build(self):
        self.cmd += " {}".format(self.image)
        return self.cmd

class PushCommandBuilder():
    cmd = None

    def __init__(self, cmd, image):
        self.cmd = cmd + " push {}".format(image)

    def set_tag(self, tag):
        self.cmd += ":{}".format(tag)
        return self

    def build(self):
        return self.cmd

class RunCommandBuilder():
    cmd = None
    image = None
    entrypoint = False
    parameters = None

    def __init__(self, cmd, image):
        self.image = image
        self.cmd = cmd + " run"

    def set_name(self, name):
        self.cmd += " --name {}".format(name)
        return self

    def set_rm(self):
        self.cmd += " --rm"
        return self

    def set_entrypoint(self, parameters):
        self.parameters = parameters
        self.entrypoint = True
        self.cmd += " --entrypoint"
        return self

    def set_gpu(self):
        self.cmd += " --gpus all --ipc=host"
        return self

    def set_mode(self, mode):
        self.cmd += " -{}".format(mode)
        return self

    def set_port(self, port):
        self.cmd += " -p {}".format(port)
        return self

    def set_volume(self, volume):
        self.cmd += " -v {}".format(volume)
        return self

    def set_params(self, params):
        self.cmd += " {}".format(params)
        return self

    def build(self, command):
        if (self.entrypoint):
            job_command = EP_COMMAND.format(command=command, image=self.image, parameters=self.parameters)
            self.cmd += job_command
        else:
            job_command = EX_COMMAND.format(command=command, image=self.image)
            self.cmd += job_command

        return self.cmd

class DockerCommandBuilder():
    cmd = None

    def __init__(self, host=None):
        self.cmd = "docker"
        if host is not None:
            print("Executing Docker on Host {}".format(host))
            self.cmd += " -H {}".format(host)

    def login(self, hub):
        self.cmd += " login {}".format(hub)
        return self.cmd

    def build(self, image):
        self.cmd += " build -t {} .".format(image)
        return self.cmd

    def tag(self, src, image, tag):
        self.cmd += " tag {src} {image}:{tag}".format(src=src, image=image, tag=tag)
        return self.cmd

    def save(self, image):
        return SaveCommandBuilder(self.cmd, image)

    def push(self, image):
        return PushCommandBuilder(self.cmd, image)

    def run(self, image):
        return RunCommandBuilder(self.cmd, image)



"""Jupyter Component"""

from schema import Schema, And, Optional
from ..objects.component import Activation, Component
from ..systems.execution import docker

HELP_TEMPLATE = "Spin up Jupyter in a Docker Container (port={port}, folder={folder})"
COMMAND_TEMPLATE = "jupyter {job} --ip=0.0.0.0 --port=8888 --allow-root --notebook-dir={folder}"

class Jupyter(Component):
    """
    Jupyter Class

    Provides the ability to spin up Jupyter notebooks inside a Docker container with the exact
    configuration as the rest of the project and it's jobs
    """

    activation = Activation.PROJECT
    commands = ["jupyter"]

    schema = Schema({
        Optional('port'): And(int, error='Jupyter \'port\' must be an Integer'),
        Optional('folder'): And(str, error='Jupyter \'folder\' must be a String'),
        Optional('mappings'): And(list, error='Jupyter \'mappings\' must be a List'),
        Optional('lab'): And(bool, error='Jupyter \'lab\' must be a Boolean'),
    }, ignore_extra_keys=True)

    port = None
    folder = None
    mappings = None
    lab = None

    def __init__(self, port=8888, folder=".", mappings=None, lab=False):
        """Initialize the class with simple default values for port and folder"""
        self.port = port
        self.folder = folder
        self.mappings = ["."] if mappings is None else mappings
        self.lab = lab

    def addParsers(self, subparsers):
        """
        SkeleParser Hook

        Adds a parser for the jupter command the starts up the notebooks in Docker
        """

        helpMessage = HELP_TEMPLATE.format(port=self.port, folder=self.folder)
        subparsers.add_parser("jupyter", help=helpMessage)
        return subparsers

    def execute(self, config, args, host=None):
        """
        Execution Hook

        Executed when the jupyter command is provided it reads from the config to spin up a volume
        mapped container on the configured port with the configured folder based on the same image
        that the rest of the project utilizes
        """

        docker.build(config, host=host, verbose=args.verbose_global)

        job = "lab" if self.lab else "notebook"
        command = COMMAND_TEMPLATE.format(job=job, folder=self.folder)
        ports = ["{port}:8888".format(port=self.port)]
        # Only map current dir if not running on remote host
        if host is None:
            mappings = ["."] + self.mappings if "." not in self.mappings else self.mappings
        else:
            mappings = self.mappings

        print("Notebook Starting on localhost:{port}".format(port=self.port))
        print("Copy the token below to authenticate with Jupyter")

        return docker.run(
            config, command, "it", ports, mappings, "jupyter", host=host,
            verbose=args.verbose_global
        )



import unittest
from skelebot.objects.skeleYaml import SkeleYaml

class TChild(SkeleYaml):

    par_one = None
    par_two = None
    nope = None

    def __init__(self, par_one: int, par_two: int):
        self.par_one = par_one
        self.par_two = par_two
        self.nope = "does not show"

    def toDict(self):
        self.nope = None
        return super().toDict()

class TParent(SkeleYaml):

    childs = None
    fav_child = None
    
    def __init__(self, childs: dict, fav_child: TChild):
        self.childs = childs
        self.fav_child = fav_child

class TestSkeleYaml(unittest.TestCase):

    def test_toDict(self):

        parent = TParent({
                "Jon": TChild(33, 1),
                "Geoffrey": TChild(45, None)
            }, TChild(12, 42))

        res = parent.toDict()
        res_childs = res["childs"]
        res_jon = res_childs["Jon"]
        res_geo = res_childs["Geoffrey"]
        res_fav = res["fav_child"]

        self.assertEqual(res_jon["par_one"], 33)
        self.assertEqual(res_jon["par_two"], 1)
        self.assertNotIn("nope", res_jon)
        self.assertEqual(res_geo["par_one"], 45)
        self.assertNotIn("par_two", res_geo)
        self.assertNotIn("nope", res_geo)
        self.assertEqual(res_fav["par_one"], 12)
        self.assertEqual(res_fav["par_two"], 42)
        self.assertNotIn("nope", res_fav)

if __name__ == '__main__':
    unittest.main()



import argparse
import unittest
from unittest import mock

import skelebot as sb

# Just a dummy component for testing
class StoreUtterance(sb.objects.component.Component):
    activation = sb.objects.component.Activation.ALWAYS
    utterance = None

    def __init__(self, utterance=None):
        self.utterance = utterance

    def scaffold(self):
        self.utterance = sb.systems.scaffolding.prompt.promptUser("Say something")
        return self

class TestComponent(unittest.TestCase):

    def setUp(self):
        self.component = StoreUtterance()

    @mock.patch('skelebot.systems.scaffolding.prompt.input')
    def test_scaffold(self, mock_input):
        mock_input.return_value = "Something awesome"

        self.component.scaffold()

        mock_input.assert_called_once_with("Say something: ")
        self.assertEqual(self.component.utterance, "Something awesome")

    def test_addParsers(self):
        parser = argparse.ArgumentParser()
        subparsers_before = parser.add_subparsers(dest="dummy")
        subparsers_after = self.component.addParsers(subparsers_before)

        self.assertEqual(subparsers_before, subparsers_after)

    def test_execute(self):
        self.assertIsNone(self.component.execute(1, 2, 3))

if __name__ == '__main__':
    unittest.main()



#try:
    #self.artifactory.execute(config, args)
    #self.fail("Exception Not Thrown")
#except RuntimeError as err:
    #self.assertEqual(str(err), "No Compatible Version Found")

import os
import unittest
from unittest import mock

import skelebot as sb

# Test plugin that says 'Duuuuuude' at the end of every command
class SayDude(sb.objects.component.Component):
    activation = sb.objects.component.Activation.ALWAYS

    def appendCommand(self, job, native):
        return "echo Duuuuuude"

class TestDockerfile(unittest.TestCase):
    path = ""

    # Get the path to the current working directory before we mock the function to do so
    def setUp(self):
        self.path = os.getcwd()
        self.maxDiff = None

    @mock.patch('os.path.expanduser')
    @mock.patch('os.getcwd')
    def test_buildDockerfile_entrypoint_exec(self, mock_getcwd, mock_expanduser):
        folderPath = "{path}/test/files".format(path=self.path)
        filePath = "{folder}/Dockerfile".format(folder=folderPath)

        mock_expanduser.return_value = "{path}/test/plugins".format(path=self.path)
        mock_getcwd.return_value = folderPath
        config = sb.systems.generators.yaml.loadConfig()
        config.primaryExe = "ENTRYPOINT"

        expectedDockerfile = """
# This Dockerfile was generated by Skelebot
# Editing this file manually is not advised as all changes will be overwritten by Skelebot

FROM skelebot/python-base:3.9
MAINTAINER Mega Man <megaman@cars.com>
WORKDIR /app
RUN ["pip", "install", "pyyaml", "artifactory", "argparse", "coverage", "pytest"]
COPY . /app
RUN rm -rf build/
RUN rm -rf dist/
ENTRYPOINT [\"bash\", \"build.sh\"]\n"""

        sb.systems.generators.dockerfile.buildDockerfile(config)

        data = None
        with open(filePath, "r", encoding="utf-8") as file:
            data = file.read()
        self.assertTrue(data is not None)
        self.assertEqual(data, expectedDockerfile)

    @mock.patch('os.path.expanduser')
    @mock.patch('os.getcwd')
    def test_buildDockerfile_entrypoint_path(self, mock_getcwd, mock_expanduser):
        folderPath = "{path}/test/files".format(path=self.path)
        filePath = "{folder}/Dockerfile".format(folder=folderPath)

        mock_expanduser.return_value = "{path}/test/plugins".format(path=self.path)
        mock_getcwd.return_value = folderPath
        config = sb.systems.generators.yaml.loadConfig()
        config.primaryExe = "ENTRYPOINT"

        config.primaryJob = "test-entrypoint-path"
        job = sb.objects.job.Job(name="test-entrypoint-path", source="jobs/dummy.py")
        config.jobs.append(job)

        expectedDockerfile = """
# This Dockerfile was generated by Skelebot
# Editing this file manually is not advised as all changes will be overwritten by Skelebot

FROM skelebot/python-base:3.9
MAINTAINER Mega Man <megaman@cars.com>
WORKDIR /app
RUN ["pip", "install", "pyyaml", "artifactory", "argparse", "coverage", "pytest"]
COPY . /app
RUN rm -rf build/
RUN rm -rf dist/
ENTRYPOINT ["python", "-u", "jobs/dummy.py"]\n"""

        sb.systems.generators.dockerfile.buildDockerfile(config)

        data = None
        with open(filePath, "r", encoding="utf-8") as file:
            data = file.read()
        self.assertTrue(data is not None)
        self.assertEqual(data, expectedDockerfile)

    @mock.patch('os.path.expanduser')
    @mock.patch('os.getcwd')
    def test_buildDockerfile_cmd_path(self, mock_getcwd, mock_expanduser):
        folderPath = "{path}/test/files".format(path=self.path)
        filePath = "{folder}/Dockerfile".format(folder=folderPath)

        mock_expanduser.return_value = "{path}/test/plugins".format(path=self.path)
        mock_getcwd.return_value = folderPath
        config = sb.systems.generators.yaml.loadConfig()
        config.primaryExe = "CMD"

        config.primaryJob = "test-cmd-path"
        job = sb.objects.job.Job(name="test-cmd-path", source="jobs/dummy.py")
        config.jobs.append(job)

        expectedDockerfile = """
# This Dockerfile was generated by Skelebot
# Editing this file manually is not advised as all changes will be overwritten by Skelebot

FROM skelebot/python-base:3.9
MAINTAINER Mega Man <megaman@cars.com>
WORKDIR /app
RUN ["pip", "install", "pyyaml", "artifactory", "argparse", "coverage", "pytest"]
COPY . /app
RUN rm -rf build/
RUN rm -rf dist/
CMD /bin/bash -c "python -u jobs/dummy.py --log info"\n"""

        sb.systems.generators.dockerfile.buildDockerfile(config)

        data = None
        with open(filePath, "r", encoding="utf-8") as file:
            data = file.read()
        self.assertTrue(data is not None)
        self.assertEqual(data, expectedDockerfile)

    @mock.patch('skelebot.systems.generators.dockerfile.call')
    @mock.patch('os.path.expanduser')
    @mock.patch('os.makedirs')
    @mock.patch('os.getcwd')
    def test_buildDockerfile_py_ca_file_error(self, mock_getcwd, mock_mkdir, mock_expanduser, mock_call):
        folderPath = "{path}/test/files".format(path=self.path)

        mock_expanduser.return_value = "{path}/test/plugins".format(path=self.path)
        mock_getcwd.return_value = folderPath
        mock_mkdir.return_value = 1
        mock_call.return_value = 1
        config = sb.systems.generators.yaml.loadConfig()
        config.dependencies.append("ca_file:cars:12345:python-pkg:ml-lib:0.1.0:prod")

        try:
            sb.systems.generators.dockerfile.buildDockerfile(config)
            self.fail("Exception Not Thrown")
        except Exception as exc:
            self.assertEqual(str(exc), "Failed to Obtain CodeArtifact Package")

    @mock.patch('skelebot.systems.generators.dockerfile.call')
    @mock.patch('os.path.expanduser')
    @mock.patch('os.makedirs')
    @mock.patch('os.getcwd')
    def test_buildDockerfile_base(self, mock_getcwd, mock_mkdir, mock_expanduser, mock_call):
        folderPath = "{path}/test/files".format(path=self.path)
        filePath = "{folder}/Dockerfile".format(folder=folderPath)

        mock_expanduser.return_value = "{path}/test/plugins".format(path=self.path)
        mock_getcwd.return_value = folderPath
        mock_mkdir.return_value = 1
        mock_call.return_value = 0
        config = sb.systems.generators.yaml.loadConfig()
        config.dependencies.append("github:github.com/repo")
        config.dependencies.append("github:https://github.com/securerepo")
        config.dependencies.append("file:libs/proj")
        config.dependencies.append("ca_file:cars:12345:python-pkg:ml-lib:0.1.0:prod")
        config.dependencies.append("req:requirements.txt")
        config.dependencies.append("proj:pyproject.toml")
        config.dependencies.append("dtable=9.0")

        expectedDockerfile = """
# This Dockerfile was generated by Skelebot
# Editing this file manually is not advised as all changes will be overwritten by Skelebot

FROM skelebot/python-base:3.9
MAINTAINER Mega Man <megaman@cars.com>
WORKDIR /app
COPY libs/proj libs/proj
COPY libs/ml_lib-0.1.0-py3-none-any.whl libs/ml_lib-0.1.0-py3-none-any.whl
COPY requirements.txt requirements.txt
RUN ["pip", "install", "pyyaml", "artifactory", "argparse", "coverage", "pytest", "git+github.com/repo", "git+https://github.com/securerepo", "/app/libs/proj", "/app/libs/ml_lib-0.1.0-py3-none-any.whl", "-r", "/app/requirements.txt", "requests", "numpy==1.22.0", "pandas~=1.1", "scikit-learn<=2.0.0 ; python_version<=\'3.6\'", "pytest ~= 6.2", "pytest-cov ~= 3.0", "fake-package == 1.2.3", "not-real", "dtable==9.0"]
COPY . /app
RUN rm -rf build/
RUN rm -rf dist/
CMD /bin/bash -c \"bash build.sh --env local --log info\"\n"""

        expectedCMD = "aws codeartifact get-package-version-asset --domain cars --domain-owner 12345 --repository python-pkg --package ml-lib --package-version 0.1.0 --profile prod --format pypi --asset ml_lib-0.1.0-py3-none-any.whl libs/ml_lib-0.1.0-py3-none-any.whl"

        sb.systems.generators.dockerfile.buildDockerfile(config)

        data = None
        with open(filePath, "r", encoding="utf-8") as file:
            data = file.read()
        mock_mkdir.assert_called_with("{folder}/libs".format(folder=folderPath), exist_ok=True)
        mock_call.assert_called_with(expectedCMD, shell=True)
        self.assertTrue(data is not None)
        self.assertEqual(data, expectedDockerfile)

    @mock.patch('os.path.expanduser')
    @mock.patch('os.getcwd')
    def test_buildDockerfile_base_versions(self, mock_getcwd, mock_expanduser):
        folderPath = "{path}/test/files".format(path=self.path)
        filePath = "{folder}/Dockerfile".format(folder=folderPath)

        mock_expanduser.return_value = "{path}/test/plugins".format(path=self.path)
        mock_getcwd.return_value = folderPath
        config = sb.systems.generators.yaml.loadConfig()
        config.dependencies.append("dtable=9.0")
        config.dependencies.append("pandas==0.25")
        config.dependencies.append("numpy~=1.17")
        config.dependencies.append("requests>= 2.2, == 2.*")
        config.dependencies.append("scipy!= 1.3.*")

        expectedDockerfile = """
# This Dockerfile was generated by Skelebot
# Editing this file manually is not advised as all changes will be overwritten by Skelebot

FROM skelebot/python-base:3.9
MAINTAINER Mega Man <megaman@cars.com>
WORKDIR /app
RUN ["pip", "install", "pyyaml", "artifactory", "argparse", "coverage", "pytest", "dtable==9.0", "pandas==0.25", "numpy~=1.17", "requests>= 2.2, == 2.*", "scipy!= 1.3.*"]
COPY . /app
RUN rm -rf build/
RUN rm -rf dist/
CMD /bin/bash -c \"bash build.sh --env local --log info\"\n"""

        sb.systems.generators.dockerfile.buildDockerfile(config)

        data = None
        with open(filePath, "r", encoding="utf-8") as file:
            data = file.read()
        self.assertTrue(data is not None)
        self.assertEqual(data, expectedDockerfile)

    @mock.patch('os.path.expanduser')
    @mock.patch('os.getcwd')
    def test_buildDockerfile_krb(self, mock_getcwd, mock_expanduser):
        folderPath = "{path}/test/files".format(path=self.path)
        filePath = "{folder}/Dockerfile".format(folder=folderPath)

        mock_expanduser.return_value = "{path}/test/plugins".format(path=self.path)
        mock_getcwd.return_value = folderPath
        config = sb.systems.generators.yaml.loadConfig()
        config.dependencies.append("github:github.com/repo")
        config.dependencies.append("file:libs/proj")
        config.dependencies.append("dtable=9.0")
        config.components.append(sb.components.kerberos.Kerberos("conf", "tab", "user"))

        expectedDockerfile = """
# This Dockerfile was generated by Skelebot
# Editing this file manually is not advised as all changes will be overwritten by Skelebot

FROM skelebot/python-krb
MAINTAINER Mega Man <megaman@cars.com>
WORKDIR /app
COPY libs/proj libs/proj
RUN ["pip", "install", "pyyaml", "artifactory", "argparse", "coverage", "pytest", "git+github.com/repo", "/app/libs/proj", "dtable==9.0"]
COPY . /app
RUN rm -rf build/
RUN rm -rf dist/
COPY conf /etc/krb5.conf
COPY tab /krb/auth.keytab
CMD /bin/bash -c \"/./krb/init.sh user && bash build.sh --env local --log info\"\n"""

        sb.systems.generators.dockerfile.buildDockerfile(config)

        data = None
        with open(filePath, "r", encoding="utf-8") as file:
            data = file.read()
        self.assertTrue(data is not None)
        self.assertEqual(data, expectedDockerfile)

    @mock.patch('os.path.expanduser')
    @mock.patch('os.getcwd')
    def test_buildDockerfile_no_command(self, mock_getcwd, mock_expanduser):
        folderPath = "{path}/test/files".format(path=self.path)
        filePath = "{folder}/Dockerfile".format(folder=folderPath)

        mock_expanduser.return_value = "{path}/test/plugins".format(path=self.path)
        mock_getcwd.return_value = folderPath
        config = sb.systems.generators.yaml.loadConfig()
        # No custom docker run command
        config.commands = []

        expectedDockerfile = """
# This Dockerfile was generated by Skelebot
# Editing this file manually is not advised as all changes will be overwritten by Skelebot

FROM skelebot/python-base:3.9
MAINTAINER Mega Man <megaman@cars.com>
WORKDIR /app
RUN ["pip", "install", "pyyaml", "artifactory", "argparse", "coverage", "pytest"]
COPY . /app
CMD /bin/bash -c \"bash build.sh --env local --log info\"\n"""

        sb.systems.generators.dockerfile.buildDockerfile(config)

        data = None
        with open(filePath, "r", encoding="utf-8") as file:
            data = file.read()
        self.assertTrue(data is not None)
        self.assertEqual(data, expectedDockerfile)

    @mock.patch('os.path.expanduser')
    @mock.patch('os.getcwd')
    def test_buildDockerfile_append_command(self, mock_getcwd, mock_expanduser):
        folderPath = "{path}/test/files".format(path=self.path)
        filePath = "{folder}/Dockerfile".format(folder=folderPath)

        mock_expanduser.return_value = "{path}/test/plugins".format(path=self.path)
        mock_getcwd.return_value = folderPath
        config = sb.systems.generators.yaml.loadConfig()
        config.language = None
        config.components.append(SayDude())

        expectedDockerfile = """
# This Dockerfile was generated by Skelebot
# Editing this file manually is not advised as all changes will be overwritten by Skelebot

FROM skelebot/python-base:3.9
MAINTAINER Mega Man <megaman@cars.com>
WORKDIR /app
RUN ["pip", "install", "pyyaml", "artifactory", "argparse", "coverage", "pytest"]
COPY . /app
RUN rm -rf build/
RUN rm -rf dist/
CMD /bin/bash -c \"bash build.sh --env local --log info && echo Duuuuuude\"\n"""

        sb.systems.generators.dockerfile.buildDockerfile(config)

        data = None
        with open(filePath, "r", encoding="utf-8") as file:
            data = file.read()
        self.assertTrue(data is not None)
        self.assertEqual(data, expectedDockerfile)

    @mock.patch('os.path.expanduser')
    @mock.patch('os.getcwd')
    def test_buildDockerfile_custom(self, mock_getcwd, mock_expanduser):
        folderPath = "{path}/test/files".format(path=self.path)
        filePath = "{folder}/Dockerfile".format(folder=folderPath)

        mock_expanduser.return_value = "{path}/test/plugins".format(path=self.path)
        mock_getcwd.return_value = folderPath
        config = sb.systems.generators.yaml.loadConfig()
        config.baseImage = "whatever:uwant"

        expectedDockerfile = """
# This Dockerfile was generated by Skelebot
# Editing this file manually is not advised as all changes will be overwritten by Skelebot

FROM whatever:uwant
MAINTAINER Mega Man <megaman@cars.com>
WORKDIR /app
RUN ["pip", "install", "pyyaml", "artifactory", "argparse", "coverage", "pytest"]
COPY . /app
RUN rm -rf build/
RUN rm -rf dist/
CMD /bin/bash -c \"bash build.sh --env local --log info\"\n"""

        sb.systems.generators.dockerfile.buildDockerfile(config)

        data = None
        with open(filePath, "r", encoding="utf-8") as file:
            data = file.read()
        self.assertTrue(data is not None)
        self.assertEqual(data, expectedDockerfile)

    @mock.patch('os.path.expanduser')
    @mock.patch('os.getcwd')
    def test_buildDockerfile_timezone(self, mock_getcwd, mock_expanduser):
        folderPath = "{path}/test/files".format(path=self.path)
        filePath = "{folder}/Dockerfile".format(folder=folderPath)

        mock_expanduser.return_value = "{path}/test/plugins".format(path=self.path)
        mock_getcwd.return_value = folderPath
        config = sb.systems.generators.yaml.loadConfig()
        config.timezone = "America/Chicago"
        config.dependencies.append("github:github.com/repo")
        config.dependencies.append("file:libs/proj")
        config.dependencies.append("dtable=9.0")

        expectedDockerfile = """
# This Dockerfile was generated by Skelebot
# Editing this file manually is not advised as all changes will be overwritten by Skelebot

FROM skelebot/python-base:3.9
MAINTAINER Mega Man <megaman@cars.com>
WORKDIR /app
ENV TZ=America/Chicago
RUN ln -snf /usr/share/zoneinfo/$TZ /etc/localtime && echo $TZ > /etc/timezone
COPY libs/proj libs/proj
RUN ["pip", "install", "pyyaml", "artifactory", "argparse", "coverage", "pytest", "git+github.com/repo", "/app/libs/proj", "dtable==9.0"]
COPY . /app
RUN rm -rf build/
RUN rm -rf dist/
CMD /bin/bash -c \"bash build.sh --env local --log info\"\n"""

        sb.systems.generators.dockerfile.buildDockerfile(config)

        data = None
        with open(filePath, "r", encoding="utf-8") as file:
            data = file.read()
        self.assertTrue(data is not None)
        self.assertEqual(data, expectedDockerfile)

if __name__ == '__main__':
    unittest.main()



import os
import copy
import unittest
from unittest import mock

import skelebot as sb
from skelebot.components.bump import Bump

TEMPLATE = {
    "dirs": ["src/assets/"],
    "files": [
        {
            "name": "src/app.py",
            "template": "files/app_py"
        },{
            "name": "src/server.py",
            "template": "files/server_py"
        },{
            "name": "src/config.py",
            "template": "files/config_py"
        },{
            "name": "src/assets/style.css",
            "template": "files/style_css"
        }
    ],
    "prompts": [
        {
            "var": "aws_prod",
            "message": "Enter AWS-PROD PROFILE"
        }
    ],
    "config": {
        "name": "{name}",
        "dependencies": ["dash~=2.0"],
        "ports": ["5000:5000"],
        "primaryJob": "run",
        "jobs": {
            "name": "run",
            "source": "src/app.py",
            "help": "Start the Dashboard"
        }
    }
}

class TestScaffolder(unittest.TestCase):

    @mock.patch('os.path.expanduser')
    @mock.patch('os.getcwd')
    @mock.patch('skelebot.systems.scaffolding.scaffolder.ComponentFactory')
    @mock.patch('skelebot.systems.scaffolding.scaffolder.promptUser')
    def test_execute_scaffold_abort(self, mock_prompt, mock_cFactory, mock_getcwd, mock_expanduser):
        mock_expanduser.return_value = "test/plugins"
        mock_prompt.side_effect = ["test", "test proj", "sean", "email", "Default", False]
        try:
            scaffolder = sb.systems.scaffolding.scaffolder.Scaffolder(existing=False)
            scaffolder.scaffold()
            self.fail("Exception Expected")
        except Exception as exc:
            self.assertEqual(str(exc), "Aborting Scaffolding Process")

            mock_prompt.assert_any_call("Enter a PROJECT NAME")
            mock_prompt.assert_any_call("Enter a PROJECT DESCRIPTION")
            mock_prompt.assert_any_call("Enter a MAINTAINER NAME")
            mock_prompt.assert_any_call("Enter a CONTACT EMAIL")
            mock_prompt.assert_any_call("Select a TEMPLATE", options=["Default", "Dash", "Git"])
            mock_prompt.assert_any_call("Confirm Skelebot Setup", boolean=True)


    @mock.patch('os.path.expanduser')
    @mock.patch('os.getcwd')
    @mock.patch('os.makedirs')
    #@mock.patch('skelebot.systems.scaffolding.scaffolder.ComponentFactory')
    @mock.patch('skelebot.systems.scaffolding.scaffolder.yaml')
    @mock.patch('skelebot.systems.scaffolding.scaffolder.promptUser')
    def test_execute_scaffold_existing_init(self, mock_prompt, mock_yaml, #mock_cFactory,
                                            mock_makedirs, mock_getcwd, mock_expanduser):
        mock_expanduser.return_value = "test/plugins"
        mock_prompt.side_effect = ["test", "test proj", "sean", "email",
                                   "Dash", True]

        scaffolder = sb.systems.scaffolding.scaffolder.Scaffolder(existing=True)
        scaffolder.scaffold()

        mock_prompt.assert_any_call("Enter a PROJECT NAME")
        mock_prompt.assert_any_call("Enter a PROJECT DESCRIPTION")
        mock_prompt.assert_any_call("Enter a MAINTAINER NAME")
        mock_prompt.assert_any_call("Enter a CONTACT EMAIL")
        mock_prompt.assert_any_call("Select a TEMPLATE", options=["Default", "Dash", "Git"])
        mock_prompt.assert_any_call("Confirm Skelebot Setup", boolean=True)

        mock_yaml.saveConfig.assert_called_once()

    @mock.patch('os.path.exists')
    @mock.patch('os.path.expanduser')
    @mock.patch('os.getcwd')
    @mock.patch('os.makedirs')
    @mock.patch('skelebot.systems.scaffolding.scaffolder.call')
    @mock.patch('skelebot.systems.scaffolding.scaffolder.open')
    @mock.patch('skelebot.systems.scaffolding.scaffolder.Config')
    @mock.patch('skelebot.systems.scaffolding.scaffolder.ComponentFactory')
    @mock.patch('skelebot.systems.scaffolding.scaffolder.dockerfile')
    @mock.patch('skelebot.systems.scaffolding.scaffolder.dockerignore')
    @mock.patch('skelebot.systems.scaffolding.scaffolder.pyyaml')
    @mock.patch('skelebot.systems.scaffolding.scaffolder.readme')
    @mock.patch('skelebot.systems.scaffolding.scaffolder.yaml')
    @mock.patch('skelebot.systems.scaffolding.scaffolder.promptUser')
    def test_execute_scaffold_git_pull(self, mock_prompt, mock_yaml, mock_readme, mock_pyyaml,
                              mock_dignore, mock_dockerfile, mock_cFactory, mock_config, mock_open,
                              mock_call, mock_makedirs, mock_getcwd, mock_expanduser, mock_exists):
        mock_exists.return_value = True

        mock_call.return_value = 0
        mock_call.configure_mock(shell=True)

        mock_expanduser.return_value = "test/plugins"
        mock_prompt.side_effect = ["test", "test proj", "sean", "email",
                                    "Git", "git@repo", "data-prod", True]

        exp_cfg = copy.deepcopy(TEMPLATE)
        mock_pyyaml.load.return_value = exp_cfg

        # Set up mock components with scaffolding
        mock_single_comp = mock.MagicMock()
        mock_single_comp.scaffold.return_value = Bump()
        mock_dict_comp = mock.MagicMock()
        mock_dict_comp.scaffold.return_value = mock_dict_comp
        mock_dict_comp.toDict.return_value = {"fake_attr": "aaaa"}
        mock_cFactory.return_value.buildComponents.return_value = [
            mock_single_comp, mock_dict_comp
        ]

        scaffolder = sb.systems.scaffolding.scaffolder.Scaffolder(existing=False)
        scaffolder.scaffold()

        mock_prompt.assert_any_call("Enter a PROJECT NAME")
        mock_prompt.assert_any_call("Enter a PROJECT DESCRIPTION")
        mock_prompt.assert_any_call("Enter a MAINTAINER NAME")
        mock_prompt.assert_any_call("Enter a CONTACT EMAIL")
        mock_prompt.assert_any_call("Select a TEMPLATE", options=["Default", "Dash", "Git"])
        mock_prompt.assert_any_call("Enter AWS-PROD PROFILE")
        mock_prompt.assert_any_call("Confirm Skelebot Setup", boolean=True)

        exp_cfg["config"]["name"] = "test"
        exp_cfg["config"]["components"] = {"magicmock": {"fake_attr": "aaaa"}}
        mock_config.load.assert_called_once_with(exp_cfg["config"])

        mock_makedirs.assert_any_call("src/assets/", exist_ok=True)
        dirname = os.path.dirname(os.path.dirname(sb.__file__))
        mock_open.assert_any_call(os.path.join(dirname, "skelebot/systems/scaffolding/templates/git_repo/files/app_py"), "r", encoding="utf-8")
        mock_open.assert_any_call("src/app.py", "w", encoding="utf-8")
        mock_open.assert_any_call(os.path.join(dirname, "skelebot/systems/scaffolding/templates/git_repo/files/server_py"), "r", encoding="utf-8")
        mock_open.assert_any_call("src/server.py", "w", encoding="utf-8")
        mock_open.assert_any_call(os.path.join(dirname, "skelebot/systems/scaffolding/templates/git_repo/files/config_py"), "r", encoding="utf-8")
        mock_open.assert_any_call("src/config.py", "w", encoding="utf-8")
        mock_open.assert_any_call(os.path.join(dirname, "skelebot/systems/scaffolding/templates/git_repo/files/style_css"), "r", encoding="utf-8")
        mock_open.assert_any_call("src/assets/style.css", "w", encoding="utf-8")

        mock_dockerfile.buildDockerfile.assert_called_once()
        mock_dignore.buildDockerignore.assert_called_once()
        mock_readme.buildREADME.assert_called_once()
        mock_yaml.saveConfig.assert_called_once()
        template_path = os.path.join(dirname, "skelebot/systems/scaffolding/templates/git_repo")
        mock_call.assert_called_once_with(f"cd {template_path} && git pull --rebase", shell=True)

    @mock.patch('os.path.exists')
    @mock.patch('os.path.expanduser')
    @mock.patch('os.getcwd')
    @mock.patch('os.makedirs')
    @mock.patch('skelebot.systems.scaffolding.scaffolder.call')
    @mock.patch('skelebot.systems.scaffolding.scaffolder.open')
    @mock.patch('skelebot.systems.scaffolding.scaffolder.Config')
    @mock.patch('skelebot.systems.scaffolding.scaffolder.ComponentFactory')
    @mock.patch('skelebot.systems.scaffolding.scaffolder.dockerfile')
    @mock.patch('skelebot.systems.scaffolding.scaffolder.dockerignore')
    @mock.patch('skelebot.systems.scaffolding.scaffolder.pyyaml')
    @mock.patch('skelebot.systems.scaffolding.scaffolder.readme')
    @mock.patch('skelebot.systems.scaffolding.scaffolder.yaml')
    @mock.patch('skelebot.systems.scaffolding.scaffolder.promptUser')
    def test_execute_scaffold_git_clone(self, mock_prompt, mock_yaml, mock_readme, mock_pyyaml,
                              mock_dignore, mock_dockerfile, mock_cFactory, mock_config, mock_open,
                              mock_call, mock_makedirs, mock_getcwd, mock_expanduser, mock_exists):
        mock_exists.return_value = False

        mock_call.return_value = 0
        mock_call.configure_mock(shell=True)

        mock_expanduser.return_value = "test/plugins"
        mock_prompt.side_effect = ["test", "test proj", "sean", "email",
                                   "Git", "git@repo", "data-prod", True]

        mock_pyyaml.load.return_value = copy.deepcopy(TEMPLATE)

        # Set up mock components with scaffolding
        mock_single_comp = mock.MagicMock()
        mock_single_comp.scaffold.return_value = Bump()
        mock_cFactory.return_value.buildComponents.return_value = [
            mock_single_comp
        ]

        scaffolder = sb.systems.scaffolding.scaffolder.Scaffolder(existing=False)
        scaffolder.scaffold()

        mock_prompt.assert_any_call("Enter a PROJECT NAME")
        mock_prompt.assert_any_call("Enter a PROJECT DESCRIPTION")
        mock_prompt.assert_any_call("Enter a MAINTAINER NAME")
        mock_prompt.assert_any_call("Enter a CONTACT EMAIL")
        mock_prompt.assert_any_call("Select a TEMPLATE", options=["Default", "Dash", "Git"])
        mock_prompt.assert_any_call("Enter AWS-PROD PROFILE")
        mock_prompt.assert_any_call("Confirm Skelebot Setup", boolean=True)

        mock_config.load.assert_called_once()

        mock_makedirs.assert_any_call("src/assets/", exist_ok=True)
        dirname = os.path.dirname(os.path.dirname(sb.__file__))
        mock_open.assert_any_call(os.path.join(dirname, "skelebot/systems/scaffolding/templates/git_repo/files/app_py"), "r", encoding="utf-8")
        mock_open.assert_any_call("src/app.py", "w", encoding="utf-8")
        mock_open.assert_any_call(os.path.join(dirname, "skelebot/systems/scaffolding/templates/git_repo/files/server_py"), "r", encoding="utf-8")
        mock_open.assert_any_call("src/server.py", "w", encoding="utf-8")
        mock_open.assert_any_call(os.path.join(dirname, "skelebot/systems/scaffolding/templates/git_repo/files/config_py"), "r", encoding="utf-8")
        mock_open.assert_any_call("src/config.py", "w", encoding="utf-8")
        mock_open.assert_any_call(os.path.join(dirname, "skelebot/systems/scaffolding/templates/git_repo/files/style_css"), "r", encoding="utf-8")
        mock_open.assert_any_call("src/assets/style.css", "w", encoding="utf-8")

        mock_dockerfile.buildDockerfile.assert_called_once()
        mock_dignore.buildDockerignore.assert_called_once()
        mock_readme.buildREADME.assert_called_once()
        mock_yaml.saveConfig.assert_called_once()
        template_path = os.path.join(dirname, "skelebot/systems/scaffolding/templates/git_repo")
        mock_call.assert_called_once_with(f"git clone git@repo {template_path}", shell=True)

    @mock.patch('os.path.expanduser')
    @mock.patch('os.getcwd')
    @mock.patch('os.makedirs')
    @mock.patch('skelebot.systems.scaffolding.scaffolder.open')
    @mock.patch('skelebot.systems.scaffolding.scaffolder.Config')
    @mock.patch('skelebot.systems.scaffolding.scaffolder.ComponentFactory')
    @mock.patch('skelebot.systems.scaffolding.scaffolder.dockerfile')
    @mock.patch('skelebot.systems.scaffolding.scaffolder.dockerignore')
    @mock.patch('skelebot.systems.scaffolding.scaffolder.pyyaml')
    @mock.patch('skelebot.systems.scaffolding.scaffolder.readme')
    @mock.patch('skelebot.systems.scaffolding.scaffolder.yaml')
    @mock.patch('skelebot.systems.scaffolding.scaffolder.promptUser')
    def test_execute_scaffold_dash(self, mock_prompt, mock_yaml, mock_readme, mock_pyyaml,
                              mock_dignore, mock_dockerfile, mock_cFactory, mock_config, mock_open,
                              mock_makedirs, mock_getcwd, mock_expanduser):
        mock_expanduser.return_value = "test/plugins"
        mock_prompt.side_effect = ["test", "test proj", "sean", "email",
                                   "Dash", "data-prod",  True]

        mock_pyyaml.load.return_value = copy.deepcopy(TEMPLATE)

        # Set up mock components with scaffolding
        mock_single_comp = mock.MagicMock()
        mock_single_comp.scaffold.return_value = Bump()
        mock_cFactory.return_value.buildComponents.return_value = [
            mock_single_comp
        ]

        scaffolder = sb.systems.scaffolding.scaffolder.Scaffolder(existing=False)
        scaffolder.scaffold()

        mock_prompt.assert_any_call("Enter a PROJECT NAME")
        mock_prompt.assert_any_call("Enter a PROJECT DESCRIPTION")
        mock_prompt.assert_any_call("Enter a MAINTAINER NAME")
        mock_prompt.assert_any_call("Enter a CONTACT EMAIL")
        mock_prompt.assert_any_call("Select a TEMPLATE", options=["Default", "Dash", "Git"])
        mock_prompt.assert_any_call("Enter AWS-PROD PROFILE")
        mock_prompt.assert_any_call("Confirm Skelebot Setup", boolean=True)

        mock_config.load.assert_called_once()

        mock_makedirs.assert_any_call("src/assets/", exist_ok=True)
        dirname = os.path.dirname(os.path.dirname(sb.__file__))
        mock_open.assert_any_call(os.path.join(dirname, "skelebot/systems/scaffolding/templates/python_dash/files/app_py"), "r", encoding="utf-8")
        mock_open.assert_any_call("src/app.py", "w", encoding="utf-8")
        mock_open.assert_any_call(os.path.join(dirname, "skelebot/systems/scaffolding/templates/python_dash/files/server_py"), "r", encoding="utf-8")
        mock_open.assert_any_call("src/server.py", "w", encoding="utf-8")
        mock_open.assert_any_call(os.path.join(dirname, "skelebot/systems/scaffolding/templates/python_dash/files/config_py"), "r", encoding="utf-8")
        mock_open.assert_any_call("src/config.py", "w", encoding="utf-8")
        mock_open.assert_any_call(os.path.join(dirname, "skelebot/systems/scaffolding/templates/python_dash/files/style_css"), "r", encoding="utf-8")
        mock_open.assert_any_call("src/assets/style.css", "w", encoding="utf-8")

        mock_dockerfile.buildDockerfile.assert_called_once()
        mock_dignore.buildDockerignore.assert_called_once()
        mock_readme.buildREADME.assert_called_once()
        mock_yaml.saveConfig.assert_called_once()

    @mock.patch('skelebot.systems.scaffolding.prompt.input')
    def test_scaffold_prompt(self, mock_input):
        mock_input.return_value = "hi"

        msg = sb.systems.scaffolding.prompt.promptUser("Enter A Message")

        mock_input.assert_called_with("Enter A Message: ")
        self.assertEqual(msg, "hi")

    @mock.patch('skelebot.systems.scaffolding.prompt.input')
    def test_scaffold_prompt_options(self, mock_input):
        mock_input.return_value = "rf"

        msg = sb.systems.scaffolding.prompt.promptUser("Select an Algorithm", options=["glm", "rf", "lgbm"])

        mock_input.assert_called_with("Select an Algorithm [glm, rf, lgbm]: ")
        self.assertEqual(msg, "rf")

    def test_scaffold_prompt_option(self):
        msg = sb.systems.scaffolding.prompt.promptUser("Select an Algorithm", options=["glm"])

        self.assertEqual(msg, "glm")

    @mock.patch('skelebot.systems.scaffolding.prompt.input')
    def test_scaffold_prompt_boolean(self, mock_input):
        mock_input.return_value = "Y"

        msg = sb.systems.scaffolding.prompt.promptUser("Would you like CHEESE with that?", boolean=True)

        mock_input.assert_called_with("Would you like CHEESE with that? [Y/N]: ")
        self.assertTrue(msg)

    @mock.patch('skelebot.systems.scaffolding.prompt.input')
    def test_scaffold_prompt_boolean_false(self, mock_input):
        mock_input.return_value = "n"

        msg = sb.systems.scaffolding.prompt.promptUser("Would you like CHEESE with that?", boolean=True)

        mock_input.assert_called_with("Would you like CHEESE with that? [Y/N]: ")
        self.assertFalse(msg)

if __name__ == '__main__':
    unittest.main()



import copy
import argparse
import unittest
from unittest import mock

from schema import SchemaError

import skelebot as sb

class TestPlugin(unittest.TestCase):

    package_yaml = {
        "path": "test.zip",
        "ignores": ["one.txt", "two.txt"]
    }

    def test_addParsers(self):
        package = sb.components.package.Package()

        parser = argparse.ArgumentParser(formatter_class=argparse.RawTextHelpFormatter)
        subparsers = parser.add_subparsers(dest="job")
        subparsers = package.addParsers(subparsers)

        self.assertNotEqual(subparsers.choices["package"], None)

    @mock.patch('skelebot.components.package.call')
    def test_execute(self, mock_call):
        mock_call.return_value = 0

        config = sb.objects.config.Config()
        args = argparse.Namespace()

        package = sb.components.package.Package(path="test.zip")
        package.execute(config, args)

        mock_call.assert_called_once_with("zip -r test.zip .", shell=True)

    @mock.patch('os.remove')
    @mock.patch('os.path.exists')
    @mock.patch('skelebot.components.package.call')
    def test_execute_existing(self, mock_call, mock_exists, mock_remove):
        mock_call.return_value = 0
        mock_exists.return_value = True

        config = sb.objects.config.Config()
        args = argparse.Namespace()

        package = sb.components.package.Package(path="test.zip")
        package.execute(config, args)

        mock_call.assert_called_once_with("zip -r test.zip .", shell=True)
        mock_exists.assert_called_once_with("test.zip")
        mock_remove.assert_called_once_with("test.zip")

    @mock.patch('skelebot.components.package.call')
    def test_execute_ignores(self, mock_call):
        mock_call.return_value = 0

        config = sb.objects.config.Config()
        args = argparse.Namespace()

        package = sb.components.package.Package(path="test.zip", ignores=[r"folder/**\*", "file.txt"])
        package.execute(config, args)

        mock_call.assert_called_once_with(r"zip -r test.zip . -x folder/**\* file.txt", shell=True)

    def test_validate_valid(self):
        try:
            sb.components.package.Package.validate(self.package_yaml)
        except:
            self.fail("Validation Raised Exception Unexpectedly")

    def validate_error(self, attr, reset, expected):
        package_yaml = copy.deepcopy(self.package_yaml)
        package_yaml[attr] = reset

        try:
            sb.components.package.Package.validate(package_yaml)
        except SchemaError as error:
            self.assertEqual(error.code, "Package '{attr}' must be a{expected}".format(attr=attr, expected=expected))

    def test_invalid(self):
        self.validate_error('path', 123, ' String')
        self.validate_error('ignores', 123, ' List')

if __name__ == '__main__':
    unittest.main()



from skelebot.objects.component import *

import idoubtthiswilleverbealegitimatepackagename

# Just a dummy plugin for testing
class AddNumbers(Component):
    activation = Activation.ALWAYS
    commands = ["addNumbers"]

    def addParsers(self, subparsers):
       subparsers.add_parser("addNumbers", help="adds the numbers 1 and 2 to get 3!")
       return subparsers

    def execute(self, config, args):
        a = 1
        b = 2
        print(a, " + ", b, " = ", a + b)



"""Job Argument Class"""

from schema import Schema, And, Optional
from .skeleYaml import SkeleYaml

class Arg(SkeleYaml):
    """
    Argument Class

    Defines a single, required, positional argument for a job in the skelebot config
    """

    schema = Schema({
        'name': And(str, error='Arg \'name\' must be a String'),
        Optional('choices'): And(list, error='Arg \'choices\' must be a List'),
        Optional('help'): And(str, error='Arg \'help\' must be a String')
    }, ignore_extra_keys=True)

    name = None
    choices = None
    help = None

    def __init__(self, name, choices=None, help=None):
        """Initialize the attributes of the argument"""

        self.name = name
        self.choices = choices
        self.help = help



from ...objects.skeleYaml import SkeleYaml

class ArtifactRepo(SkeleYaml):
    """
    Artifact Repo Interface

    Defines the essential functions for an Artifact Repo to implement
    """

    ERROR_NOT_COMPATIBLE = "No Compatible Version Found"
    ERROR_ALREADY_PUSHED = "This artifact version already exists. Please bump the version or use the force parameter (-f) to overwrite the artifact."

    def push(self, artifact, version, force=False, user=None, token=None, prefix=None):
        """ Function to be defined for pushing an artifact to an artifact repository """
        pass

    def pull(self, artifact, version, currentVersion=None, override=False, user=None, token=None):
        """ Function to be defined for pulling an artifact from an artifact repository """
        pass



"""Common Global Variables"""

from importlib import metadata

from colorama import Fore, Style

VERSION = metadata.version("skelebot")
DESCRIPTION = Style.BRIGHT + "{project}" + Style.RESET_ALL + """
{desc}
-----------------------------------
Version: {pVersion}
Environment: {env}
Skelebot Version: {version}
-----------------------------------"""
SKELEBOT_HOME = "~/.skelebot"
PLUGINS_HOME = "{home}/plugins".format(home=SKELEBOT_HOME)
PLUGINS_QUARANTINE = "{home}/plugins-quarantine".format(home=SKELEBOT_HOME)

LANGUAGE_IMAGE = {
    "base": "skelebot/python-base:{pythonVersion}",
    "krb": "skelebot/python-krb"
}

PYTHON_VERSIONS = ['3.9', '3.10', '3.11', '3.12']
DEPRECATED_VERSIONS = []

TEMPLATE_PATH = "templates/{name}"
TEMPLATES =  {
    "Default": "python",
    "Dash": "python_dash",
    "Git": "git"
}

EXT_COMMAND = {"py":"python -u ", "R":"Rscript ", "sh":"bash ", "None":""}

ERROR_HEADER = Fore.RED + "ERROR" + Style.RESET_ALL + " | "
ERROR = ERROR_HEADER + "{}"
SCHEMA_ERROR = ERROR_HEADER + "skelebot.yaml | {}"

WARN_HEADER = Fore.YELLOW + "WARN" + Style.RESET_ALL + " | "
DEPRECATION_WARNING = WARN_HEADER + "The {code} has been deprecated as of v{version}. {msg}"

INFO_HEADER = Fore.GREEN + "INFO" + Style.RESET_ALL + " | "
INFO = INFO_HEADER + "{}"



"""Prompt Function"""

def promptUser(message, options=None, boolean=False, deprecated_options=None):
    """Construct dynamic prompts for user input and return the user-entered value"""

    inp = None
    if deprecated_options is None:
        deprecated_options = []

    if (options):
        options = list(map(str, options))
        p_options = []
        for o in options:
            p_options.append(o + " (DEPRECATED)" if o in deprecated_options else o)
        if (len(options) == 1):
            inp = options[0]
        while inp not in options:
            inp = input("{msg} [{options}]: ".format(msg=message, options=', '.join(p_options)))
    elif (boolean):
        inp = input("{msg} [Y/N]: ".format(msg=message)) in ["Y", "y"]
    else:
        inp = input("{msg}: ".format(msg=message))

    return inp



import argparse
import unittest
from unittest import mock

import skelebot as sb

class TestPrime(unittest.TestCase):

    def test_addParsers(self):
        parser = argparse.ArgumentParser(formatter_class=argparse.RawTextHelpFormatter)
        subparsers = parser.add_subparsers(dest="prime")
        prime = sb.components.prime.Prime()
        subparsers = prime.addParsers(subparsers)

        self.assertNotEqual(subparsers.choices["prime"], None)

    @mock.patch('skelebot.components.prime.docker')
    def test_execute(self, mock_docker):
        mock_docker.build.return_value = 0
        mock_docker.save.return_value = 0

        config = sb.objects.config.Config()
        args = argparse.Namespace(output=None, verbose_global=False)

        prime = sb.components.prime.Prime()
        prime.execute(config, args)

        mock_docker.build.assert_called_with(config, host=None, verbose=False)

    @mock.patch('skelebot.components.prime.docker')
    def test_execute_host(self, mock_docker):
        mock_docker.build.return_value = 0
        mock_docker.save.return_value = 0

        config = sb.objects.config.Config()
        args = argparse.Namespace(output=None, verbose_global=False)

        prime = sb.components.prime.Prime()
        prime.execute(config, args, host='host1')

        mock_docker.build.assert_called_with(config, host='host1', verbose=False)

    @mock.patch('skelebot.components.prime.docker')
    def test_execute_output(self, mock_docker):
        mock_docker.build.return_value = 0
        mock_docker.save.return_value = 0

        config = sb.objects.config.Config()
        args = argparse.Namespace(output="my-image.img", verbose_global=False)

        prime = sb.components.prime.Prime()
        prime.execute(config, args)

        mock_docker.build.assert_called_with(config, host=None, verbose=False)
        mock_docker.save.assert_called_with(config, "my-image.img", host=None, verbose=False)

    @mock.patch('skelebot.components.prime.docker')
    def test_execute_exception(self, mock_docker):
        mock_docker.build.return_value = 0
        mock_docker.save.return_value = 1

        config = sb.objects.config.Config()
        args = argparse.Namespace(output="my-image.img", verbose_global=False)

        prime = sb.components.prime.Prime()
        try:
            prime.execute(config, args)
            self.fail("Exception Expected")
        except Exception as exc:
            self.assertEqual(str(exc), "Priming Failed")
            mock_docker.build.assert_called_with(config, host=None, verbose=False)
            mock_docker.save.assert_called_with(config, "my-image.img", host=None, verbose=False)

if __name__ == '__main__':
    unittest.main()



"""Component Object"""

from enum import Enum
from .skeleYaml import SkeleYaml

class Activation(Enum):
    """
    Activation Enum

    The activation value defines when a component will be available for use in a Skelebot project

    EMPTY   | Available only when executing Skelebot outside of a project
    CONFIG  | Available only when the component config is inside the skelebot.yaml
    PROJECT | Available when skelebot is run from inside any project, regardless of config
    ALWAYS  | Always available
    """

    EMPTY = 1
    CONFIG = 2
    PROJECT = 3
    ALWAYS = 4

class Component(SkeleYaml):
    """
    Component Class

    This class acts as an abstract class that defines the default, do nothing, behavior for the
    various hooks in the Skelebot Systems
    """

    activation = Activation.CONFIG
    commands = []

    # Scaffolding System Hooks
    def scaffold(self):
        """Defines the default for adding scaffolding functionality to a component"""
        return None

    # Parsing System Hooks
    def addParsers(self, subparsers):
        """Defines the default for adding subparsers to the SkeleParser"""
        return subparsers

    # Generator System Hooks
    def appendDockerignore(self):
        """Defines the default appending contents to the .dockerignore file"""
        return ""
    def appendDockerfile(self):
        """Defines the default appending contents to the Dockerfile"""
        return ""

    # Execution System Hooks
    def execute(self, config, args, host=None):
        """Defines the execution of commands in the component's commands attribute"""
        return None
    def prependCommand(self, job, native):
        """Defines the contents that are prepended to the command from the component"""
        return None
    def appendCommand(self, job, native):
        """Defines the contents that are appended to the command from the component"""
        return None
    def addDockerRunParams(self):
        """Defines the docker run parameters that are added by the component"""
        return None



import unittest
import runpy
from unittest import mock

from schema import SchemaError
from colorama import Fore, Style

import skelebot as sb

class TestSkelebotMain(unittest.TestCase):

    @mock.patch('skelebot.systems.execution.executor.execute')
    @mock.patch('skelebot.systems.generators.yaml.loadConfig')
    @mock.patch('skelebot.systems.parsing.skeleParser.SkeleParser')
    def test_skelebot_main(self, mock_parser, mock_yaml, mock_executor):
        mock_parser.return_value = "parser"
        mock_yaml.return_value = "config"
        mock_executor.return_value = 0

        sb.main()

        mock_yaml.assert_called_once_with(None)
        mock_parser.assert_called_once_with("config", None)
        mock_executor.assert_called_once_with("config", "parser")

    @mock.patch('skelebot.systems.execution.executor.execute')
    @mock.patch('skelebot.systems.generators.yaml.loadConfig')
    @mock.patch('skelebot.systems.parsing.skeleParser.SkeleParser')
    def test_skelebot_main_env(self, mock_parser, mock_yaml, mock_executor):
        mock_parser.return_value = "parser"
        mock_yaml.return_value = "config"
        mock_executor.return_value = 0

        with mock.patch("sys.argv", ["path/to/skelebot", "-e", "dev"]):
            sb.main()

        mock_yaml.assert_called_once_with("dev")
        mock_parser.assert_called_once_with("config", "dev")
        mock_executor.assert_called_once_with("config", "parser")

    @mock.patch('skelebot.systems.execution.executor.execute')
    @mock.patch('skelebot.systems.generators.yaml.loadConfig')
    @mock.patch('skelebot.systems.parsing.skeleParser.SkeleParser')
    def test_skelebot_main_job_env(self, mock_parser, mock_yaml, mock_executor):
        mock_parser.return_value = "parser"
        mock_yaml.return_value = "config"
        mock_executor.return_value = 0

        with mock.patch("sys.argv", ["skelebot", "train", "-e", "dev"]):
            sb.main()

        mock_yaml.assert_called_once_with(None)
        mock_parser.assert_called_once_with("config", None)
        mock_executor.assert_called_once_with("config", "parser")

    @mock.patch('skelebot.systems.execution.executor.execute')
    @mock.patch('skelebot.systems.generators.yaml.loadConfig')
    @mock.patch('skelebot.systems.parsing.skeleParser.SkeleParser')
    def test_skelebot_main_job_env_skip_build(self, mock_parser, mock_yaml, mock_executor):
        mock_parser.return_value = "parser"
        mock_yaml.return_value = "config"
        mock_executor.return_value = 0

        with mock.patch("sys.argv", ["skelebot", "-s", "-n", "train", "-e", "dev"]):
            sb.main()

        mock_yaml.assert_called_once_with(None)
        mock_parser.assert_called_once_with("config", None)
        mock_executor.assert_called_once_with("config", "parser")

    @mock.patch('builtins.print')
    @mock.patch('sys.exit')
    @mock.patch('skelebot.systems.generators.yaml.loadConfig')
    def test_skelebot_schema_error(self, mock_yaml, exit_mock, print_mock):
        mock_yaml.side_effect = SchemaError("Validation Failed")

        sb.main()

        print_mock.assert_called_once_with(Fore.RED + "ERROR" + Style.RESET_ALL + " | skelebot.yaml | Validation Failed")
        exit_mock.assert_called_once_with(1)

    @mock.patch('builtins.print')
    @mock.patch('sys.exit')
    @mock.patch('skelebot.systems.generators.yaml.loadConfig')
    def test_skelebot_runtime_error(self, mock_yaml, exit_mock, print_mock):
        mock_yaml.side_effect = RuntimeError("Environment Not Found")

        sb.main()

        print_mock.assert_called_once_with(Fore.RED + "ERROR" + Style.RESET_ALL + " | Environment Not Found")
        exit_mock.assert_called_once_with(1)


    @mock.patch('skelebot.skelebot.main')
    def test_skelebot_init(self, mock_main):
        runpy.run_module('skelebot.__init__', run_name='__main__')
        mock_main.assert_called_once_with()



"""Dockerignore Generator"""

import os

FILE_PATH = "{path}/.dockerignore"
DOCKERIGNORE = """
# This dockerignore was generated by Skelebot
# Editing this file manually is not advised as all changes will be overwritten by Skelebot

"""


def buildDockerignore(config):
    """Builds the .dockerignore file based on values from the Config object"""

    docker = DOCKERIGNORE

    # Add the base level ignores for the project defined in the yaml
    for ignore in config.ignores:
        docker += "{ignore}\n".format(ignore=ignore)

    # Pull in any additional dockerignore updates from the components
    for component in config.components:
        docker += component.appendDockerignore()

    dockerignore = open(FILE_PATH.format(path=os.getcwd()), "w", encoding="utf-8")
    dockerignore.write(docker)
    dockerignore.close()



"""Param Class"""

from schema import Schema, And, Optional
from .skeleYaml import SkeleYaml

class Param(SkeleYaml):
    """
    Param Class

    The Param Class defines a single parameter that can be passed to a job in a Skelebot project
    """

    schema = Schema({
        'name': And(str, error='Param \'name\' must be a String'),
        Optional('alt'): And(str, error='Param \'alt\' must be a String'),
        Optional('accepts'): And(str, error='Param \'accepts\' must be a String'),
        Optional('choices'): And(list, error='Param \'choices\' must be a List'),
        Optional('help'): And(str, error='Param \'help\' must be a String')
    }, ignore_extra_keys=True)

    name = None
    alt = None
    accepts = None
    default = None
    choices = None
    help = None

    def __init__(self, name, alt=None, accepts=None, default=None, choices=None, help=None):
        """Initialize the param object with all provided optional attributes"""

        self.name = name
        self.alt = alt
        self.accepts = accepts
        self.default = default
        self.choices = choices
        self.help = help



"""Package Component"""

import os
from subprocess import call
from schema import Schema, And, Optional
from ..objects.component import Activation, Component

HELP_TEMPLATE = "Package the codebase into a single zip file ({path})"
COMMAND_TEMPLATE = "zip -r {path} .{ignores}"

class Package(Component):
    """
    Package Class

    Provides the ability to package the codebase into a zip file artifact for easy distribution
    """

    activation = Activation.PROJECT
    commands = ["package"]

    schema = Schema({
        'path': And(str, error='Package \'path\' must be a String'),
        Optional('ignores'): And(list, error='Package \'ignores\' must be a List')
    }, ignore_extra_keys=True)

    path = None
    ignores = None

    def __init__(self, path=None, ignores=None):
        """Initialize the class with simple default values for path and ignores"""

        self.path = path
        self.ignores = ignores

    def addParsers(self, subparsers):
        """
        SkeleParser Hook

        Adds a parser for the package command that zips up the codebase
        """

        helpMessage = HELP_TEMPLATE.format(path=self.path)
        subparsers.add_parser("package", help=helpMessage)
        return subparsers

    def execute(self, config, args, host=None):
        """
        Execution Hook

        Executed when the package command is provided it zips the codebase into a single file
        while ingoring a list of folders and files in the process
        """
        # Ensure we always create a new zipfile
        if os.path.exists(self.path):
            os.remove(self.path)

        ignores = f" -x {' '.join(self.ignores)}" if (self.ignores is not None) else ""
        command = COMMAND_TEMPLATE.format(path=self.path, ignores=ignores)
        return call(command, shell=True)



"""Environments Component"""

import os
import re

from ..objects.component import Activation, Component

HELP_MESSAGE = "Display the available environments for the project"

class Environments(Component):
    """
    Envs Class

    Provides the ability to inspect the available environments for the project.
    """

    activation = Activation.PROJECT
    commands = ["envs"]

    def addParsers(self, subparsers):
        """
        SkeleParser Hook

        Adds a parser for the envs command to list the available skelebot environments.
        """

        subparsers.add_parser("envs", help=HELP_MESSAGE)
        return subparsers

    def execute(self, config, args, host=None):
        """
        Execution Hook

        Executed when the envs command is provided it reads the skelebot files from the local
        project to determine the list of available environments.
        """

        envs = ["[default]"]

        # Read envs from disk
        for fl in os.listdir():
            if re.match("skelebot-\\w+[.]yaml", fl):
                fl = fl.split("-")[1]
                fl = fl.split(".")[0]
                envs.append(fl)

        # Print the list of envs
        for env in envs:
            print(env)

        return 0



"""Docker Execution"""

import os
from subprocess import call
from .dockerCommand import DockerCommandBuilder
from ...systems.generators import dockerfile
from ...systems.generators import dockerignore
from ...common import INFO, WARN_HEADER

AWS_LOGIN_CMD = "$(aws ecr get-login --no-include-email --region {region} --profile {profile})"
AWS_LOGIN_CMD_V2 = "aws ecr get-login-password --region {region} --profile {profile} | docker{docker_host} login --username AWS --password-stdin {host}"

def execute(cmd, err_msg="Docker Command Failed", verbose=False):
    if verbose:
        print(INFO.format(cmd))
    status = call(cmd, shell=True)
    if (status != 0):
        raise Exception(err_msg)

    return status

def login(host=None, docker_host=None, verbose=False):
    """Login to the given Docker Host"""

    host = host if host is not None else ""
    loginCMD = DockerCommandBuilder(host=docker_host).login(host)

    status = execute(loginCMD, err_msg="Docker Login Failed", verbose=verbose)

    return status

def loginAWS(host=None, region=None, profile=None, docker_host=None, verbose=False):
    """Login to AWS with ~/.aws credentials to access an ECR host"""

    host = host if host is not None else ""
    region = region if region is not None else "us-east-1"
    profile = profile if profile is not None else "default"

    try:
        v2_docker_host = " -H {}".format(docker_host) if docker_host is not None else  ""
        loginCMD = AWS_LOGIN_CMD_V2.format(
            region=region, profile=profile, docker_host=v2_docker_host, host=host
        )

        status = execute(loginCMD, err_msg="Docker Login V2 Failed", verbose=verbose)
    # If AWS CLI V2 authentication failed try V1 command...
    except Exception:
        if docker_host is not None:
            raise ValueError("Remote hosts are not supported by Docker Login V1")
        loginCMD = AWS_LOGIN_CMD.format(region=region, profile=profile)

        status = execute(loginCMD, err_msg="Docker Login V1 Failed", verbose=verbose)

    return status

def build(config, host=None, verbose=False):
    """Build the Docker Image after building the Dockerfile and .dockerignore from Config"""

    # Build Dockerfile, .dockerignore, and Docker Image
    dockerfile.buildDockerfile(config)
    dockerignore.buildDockerignore(config)

    buildCMD = DockerCommandBuilder(host=host).build(config.getImageName())

    try:
        status = execute(buildCMD, err_msg="Docker Build Failed", verbose=verbose)
    finally:
        # Remove Files if ephemeral is set to True in Config
        if (config.ephemeral):
            os.remove("Dockerfile")
            os.remove(".dockerignore")

    return status

def run(config, command, mode, ports, mappings, task, host=None, verbose=False):
    """Run the Docker Container from the Image with the provided command"""

    image = config.getImageName()
    run_name = "{image}-{job}".format(image=image, job=task)
    runCMD = DockerCommandBuilder(host=host).run(image).set_name(run_name).set_rm().set_mode(mode)

    if config.gpu:
        runCMD = runCMD.set_gpu()

    # Construct the port mappings
    if (ports):
        for port in ports:
            runCMD = runCMD.set_port(port)

    # Construct the volume mappings
    if (mappings):
        for vmap in mappings:
            if ("~" in vmap):
                vmap = vmap.replace("~", os.path.expanduser("~"))

            if (":" not in vmap):
                vmap = "{pwd}/{vmap}:/app/{vmap}".format(pwd=os.getcwd(), vmap=vmap)

            runCMD = runCMD.set_volume(vmap)

    # Construct the additional parameters from the components
    for component in config.components:
        addParams = component.addDockerRunParams()
        if (addParams is not None):
            runCMD = runCMD.set_params(addParams)

    # Assuming the image was built without errors, run the container with the given command
    if config.primaryExe == "ENTRYPOINT":
        commandParts = command.split(" ")
        command = commandParts.pop(0)
        parameters = " ".join(commandParts)
        runCMD = runCMD.set_entrypoint(parameters)

    return execute(runCMD.build(command), verbose=verbose)

def save(config, filename="image.img", host=None, verbose=False):
    """Save the Image File to the disk"""

    cmd = DockerCommandBuilder(host=host).save(config.getImageName()).set_output(filename).build()
    return execute(cmd, verbose=verbose)

def push(config, host=None, port=None, user=None, tags=None, docker_host=None, verbose=False, omit_latest=False, omit_version=False):
    """Tag with version and latest and push the project Image to the provided Docker Image Host"""

    imageName = config.getImageName()
    port = ":{port}".format(port=port) if port is not None else ""
    host = "{host}{port}/".format(host=host, port=port) if host is not None else ""
    user = "{user}/".format(user=user) if user is not None else ""
    image = "{host}{user}{name}".format(host=host, user=user, name=imageName)

    tags = [] if tags is None else tags
    if (omit_version == False):
        tags = tags + [config.version]
    if (omit_latest == False):
        tags = tags + ["latest"]

    status = 0

    if (len(tags) == 0):
        print(WARN_HEADER + "No Tags to Publish")

    for tag in tags:
        status = execute(
            DockerCommandBuilder(host=docker_host).tag(imageName, image, tag),
            verbose=verbose
        )
        status = execute(
            DockerCommandBuilder(host=docker_host).push(image).set_tag(tag).build(),
            verbose=verbose
        )

    return status



import argparse
import os
import unittest
from unittest import mock

import skelebot as sb

class TestBump(unittest.TestCase):
    path = ""

    # Get the path to the current working directory before we mock the function to do so
    def setUp(self):
        self.path = os.getcwd()

    def test_addParsers(self):
        bump = sb.components.bump.Bump()

        parser = argparse.ArgumentParser(formatter_class=argparse.RawTextHelpFormatter)
        subparsers = parser.add_subparsers(dest="job")
        subparsers = bump.addParsers(subparsers)

        self.assertNotEqual(subparsers.choices["bump"], None)

    @mock.patch('os.path.expanduser')
    @mock.patch('os.getcwd')
    def test_execute_major(self, mock_getcwd, mock_expanduser):
        mock_expanduser.return_value = "{path}/test/plugins".format(path=self.path)
        mock_getcwd.return_value = "{path}/test/files".format(path=self.path)

        config = sb.systems.generators.yaml.loadConfig()
        args = argparse.Namespace(version='major')

        bump = sb.components.bump.Bump()
        bump.execute(config, args)

        bumpVersion = sb.systems.generators.yaml.loadVersion()
        sb.systems.generators.yaml.saveVersion("6.6.6")

        self.assertEqual(bumpVersion, "7.0.0")

    @mock.patch('os.path.expanduser')
    @mock.patch('os.getcwd')
    def test_execute_minor(self, mock_getcwd, mock_expanduser):
        mock_expanduser.return_value = "{path}/test/plugins".format(path=self.path)
        mock_getcwd.return_value = "{path}/test/files".format(path=self.path)

        config = sb.systems.generators.yaml.loadConfig()
        args = argparse.Namespace(version='minor')

        bump = sb.components.bump.Bump()
        bump.execute(config, args)

        bumpVersion = sb.systems.generators.yaml.loadVersion()
        sb.systems.generators.yaml.saveVersion("6.6.6")

        self.assertEqual(bumpVersion, "6.7.0")


    @mock.patch('os.path.expanduser')
    @mock.patch('os.getcwd')
    def test_execute_patch(self, mock_getcwd, mock_expanduser):
        mock_expanduser.return_value = "{path}/test/plugins".format(path=self.path)
        mock_getcwd.return_value = "{path}/test/files".format(path=self.path)

        config = sb.systems.generators.yaml.loadConfig()
        args = argparse.Namespace(version='patch')

        bump = sb.components.bump.Bump()
        bump.execute(config, args)

        bumpVersion = sb.systems.generators.yaml.loadVersion()
        sb.systems.generators.yaml.saveVersion("6.6.6")

        self.assertEqual(bumpVersion, "6.6.7")

if __name__ == '__main__':
    unittest.main()



"""SkeleYaml Base Object"""

import copy

class SkeleYaml():
    """
    SkeleYaml Class

    This class acts as a base for any object that needs to be converted to a Dict for YAML file
    generation purposes
    """

    schema = None

    @classmethod
    def loadList(cls, config):
        """Iterates over a list of Dicts that represent SkeleYamls and loads them into a list"""

        objs = []
        if isinstance(config, list):
            for element in config:
                objs.append(cls.load(element))

        return objs

    @classmethod
    def load(cls, config):
        """Defines the default manner of loading a Component object from a config dict"""

        cls.validate(config)
        return cls(**config)

    @classmethod
    def validate(cls, config):
        """Executes the schema validation based on the defined Schema for the implemented class"""

        if cls.schema is not None:
            cls.schema.validate(config)

    def toDict(self):
        """Convert the object attributes into a Dict structure that is ready for YAML generation"""

        dct = copy.deepcopy(vars(self))
        removeList = []
        for key, value in dct.items():
            if (value is None) or (value == []):
                removeList.append(key)
            elif (isinstance(value, SkeleYaml)):
                dct[key] = value.toDict()
            elif (isinstance(value, list)):
                dctList = []
                for element in value:
                    dctList.append(element.toDict() if isinstance(element, SkeleYaml) else element)
                dct[key] = dctList
            elif (isinstance(value, dict)):
                dctDict = {}
                for item_key, item_value in value.items():
                    dctDict[item_key] = item_value.toDict() if (isinstance(item_value, SkeleYaml)) else item_value
                dct[key] = dctDict

        for key in removeList:
            del dct[key]

        return dct
